/*
Cripple Mr. Onion - A Discworld-themed card game for the Arduboy platform.

Written in 2017 by Christopher Fenton (chris@chrisfenton.com)

To the extent possible under law, the author(s) have dedicated all copyright and related and neighboring rights to this software to the public domain worldwide. This software is distributed without any warranty.

To see the CC0 Public Domain Dedication that goes along with this software, see <http://creativecommons.org/publicdomain/zero/1.0/>. 
*/

#include <Arduboy2.h>
#include <EEPROM.h>

// make an instance of arduboy used for many functions
Arduboy2 arduboy;




/*
Cripple Mr Onion (from http://discworld.wikia.com/wiki/Cripple_Mr_Onion)

The most notable aspect of the game is that it requires an eight-suited card deck. 
The game divides the eight suits into pairs: spades and axes, clubs and tridents, 
hearts and roses, diamonds and doves. Two "normal" decks may be used, providing 
there is some way of distinguishing the "modifier" cards. (These represent the 
eight Minor Arcana suits of a Discworld "Caroc" deck: staves, swords, cups, coins, 
octagrams, elephants, terrapins and crowns).

How to play

Each player receives a hand of ten cards: five cards are dealt face-down to each 
player, and the player may then discard up to four of them, receiving new cards to 
replace them. Then a further five cards are dealt face-up to each player except the 
dealer, who receives his face-down. The first player begins by assembling his or her 
cards into one of the winning groupings described below, and displaying them. The 
next player must then create a more valuable grouping or fold. If the player is 
successful in creating a more valuable grouping, the original first player may try 
again to create an even more valuable grouping for himself, or fold. This process 
passes left around the table until only one player remains, who then wins the hand.

Scoring

The categories of winning group, in ascending order of value, are as follows. Number 
cards are worth their face value, picture cards are worth ten, and aces are worth one 
or eleven at the player's choice (a la Blackjack).

    Bagel, two cards with values totalling 20; = 1
    
    Two-card Onion, two cards with values totalling 21; = 2
    
    Broken Flush, a set of three or more cards totalling between 16 and 21 inclusive, 
    and with all but one in the same suit-pair; = 3
    
    Three-card Onion, three cards with values totalling 21; = 4
    
    Flush, as a broken flush but with all the cards being in the same suit-pair; = 5
    
    Four-card Onion, as two- and three-card Onion above; = 6
    
    Broken Royal, the combination 678 of any suit; = 7
    
    Five-card Onion; = 8 
    
    Royal, the combination 777 of any suit; = 9
    
    Six-card Onion; = 10
    
    Wild Royal, the combination 888 in a hand when eights are wild (see below); = 11
    
    Seven-card Onion; = 12
    
    Double Onion, two picture cards and two aces; = 13
    
    Triple Onion, three picture cards and three aces; = 14
    
    Lesser Onion, four picture cards and four aces; = 15
    
    Great Onion, five picture cards and five aces. = 16

Game modifiers

"Modifiers" may also be played to increase the value of a hand. Apart from the crippling 
rule, modifiers are optional rules, which may or may not be included in a game. The 
available modifiers (many of which are named after Discworld characters or concepts) are
as follows:

    Crippling Mr Onion: if a player displays a Great Onion, an opponent may display a 
    nine-card running flush and instantly win the hand. If a player displays a Great 
    or Lesser Onion, an opponent may display a ten-card running flush and instantly win 
    the hand; a player with a ten-card running flush can also use it to steal the win 
    from someone who has previously crippled Mr Onion with a nine-card running flush. 
    (This is the only non-optional modifier.)
    
    Null Eights: in a normal hand, eights may be played as if their value were zero 
    (but can be still be played with value eight if the player wishes). Thus they can 
    be included in an existing Onion in order to improve its size by one card. Whenever 
    this is done, eights become wild cards in the following hand, and this modifier cannot be
    used in that hand. After one hand with eights as wild cards, they revert to normal, and 
    this modifier becomes available again.
    
    Wild Crippling: when eights are wild, you cannot Cripple Mr Onion if your running flush 
    contains more wild eights than the Lesser or Great Onion you are trying to cripple.
    
    Octavo: when eights are wild, the grouping 88888888 is considered a Lesser Onion, but beats
    any other Lesser Onion and is considered a Great Onion for the purpose of being crippled.
    
    The Lady: a player may reveal the Queen of Spades for one of two effects: if eights are not 
    wild in the hand, the player may draw two cards from the deck, then choose one of these cards 
    to replace the queen in their hand. If eights are wild, the player can force every opponent 
    to devalue one ace in their hand to value 1 (rather than 11). The opponent chooses which ace 
    is devalued.
    
    Fate: if the Lady has been played and replaced with another card from the deck as above, the 
    King of Cups may be revealed and replaced in the same way, also rendering all Aces held by 
    the player who played the Lady unplayable. If eights are wild, the King of Cups may be played 
    to immediately cause them to cease being wild; but if played this way, any other (not the same!) 
    player who holds the Queen of Spades may reveal it to cause their eights to remain wild.

    Great A'Tuin: a player who reveals the Queen of Coins may subtract eight from the value of one 
    of their cards and add it to the value of another. Card values must still range from 1 to 11.
    
    The Elephants: a player who reveals a set of four cards, each either a nine or ten (or a wild eight),
    plus the Queen of Coins, may shift points of value between their cards to create a Double Onion, 
    and may consider any other nines or tens in their hands as ones (not aces) and twos respectively. 
    However, any other Double Onion beats this one.
    
    The Sender of Eights: displaying a Jack of Diamonds when eights are not wild causes the aces of any 
    other player who has used a Null Eight to become unplayable. When eights are wild, displaying a Jack 
    of Diamonds makes all aces unplayable and bans wild eights from taking value 1 or 11.
    
    Death: displaying a King of Swords "kills" one picture card in the hand of every player who has more 
    than one in their hand. A "killed" picture card may not participate in a Double Onion, and if eights 
    are wild may not participate in a Triple Onion either, but may still participate in other groupings.
    
    The Archchancellor: When played, the Jack of Staves is wild in all hands. However, any player who 
    plays a Jack of Staves must play all their eights as null eights. Further, any opponent may reveal 
    a King of Swords after the Jack of Staves is revealed to cause the Jack of Staves to cease being 
    wild and also cause all other players to reveal a card they have not yet revealed.
    
    The Fool: If the Jack of Clubs is declared by any player before the first player has played their 
    first group of cards, Bagels and Onions switch places in score value. Thus, Double Bagels, Triple 
    Bagels, etc. become the most valuable hands, with the exception that a Great Onion will still beat 
    a Great Bagel. It also, of course, becomes possible for a player to Cripple Mr Bagel. 
 */

//title screen graphic
const unsigned char title_screen[] PROGMEM  = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x3E, 0x00, 0x10, 0x00, 0x00, 
0x0C, 0x00, 0x00, 0x18, 0x10, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x7E, 0x00, 0x38, 0x00, 0x00, 
0x0C, 0x00, 0x00, 0x18, 0x38, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0xE6, 0x00, 0x00, 0x00, 0x00, 
0x0C, 0x00, 0x00, 0x1C, 0x38, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x01, 0x82, 0x00, 0x00, 0x00, 0x00, 
0x0C, 0x00, 0x00, 0x3C, 0x38, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x03, 0x80, 0x4E, 0x11, 0xB8, 0x6F, 
0x0C, 0x1E, 0x00, 0x3C, 0x78, 0x67, 0x00, 0x00, 
0x00, 0x00, 0x03, 0x00, 0x7F, 0x31, 0xFC, 0x7F, 
0x0C, 0x7F, 0x00, 0x3C, 0x6C, 0x7F, 0x00, 0x00, 
0x00, 0x00, 0x06, 0x00, 0xF3, 0x31, 0xC6, 0x71, 
0x8C, 0x61, 0x00, 0x36, 0x6C, 0x31, 0x80, 0x00, 
0x00, 0x00, 0x06, 0x00, 0x61, 0x31, 0x86, 0x61, 
0x98, 0xC7, 0x00, 0x66, 0x6C, 0x61, 0x00, 0x00, 
0x00, 0x00, 0x06, 0x00, 0xC0, 0x11, 0x86, 0x60, 
0x8C, 0x9E, 0x00, 0x66, 0xCC, 0x60, 0x00, 0x00, 
0x00, 0x00, 0x06, 0x00, 0x40, 0x31, 0x86, 0x21, 
0xC8, 0xF0, 0x00, 0x66, 0xC4, 0x60, 0x00, 0x00, 
0x00, 0x00, 0x06, 0x04, 0xC0, 0x31, 0x86, 0x61, 
0x8C, 0xE0, 0x00, 0x63, 0x8E, 0x60, 0x00, 0x00, 
0x00, 0x00, 0x07, 0x1E, 0x40, 0x31, 0x8E, 0x61, 
0x98, 0xC1, 0x80, 0xE3, 0x86, 0x20, 0x00, 0x00, 
0x00, 0x00, 0x03, 0xF8, 0xE0, 0x31, 0xFC, 0x3F, 
0x0C, 0x7F, 0x80, 0xC3, 0x86, 0x60, 0x00, 0x00, 
0x00, 0x00, 0x00, 0xE0, 0x40, 0x11, 0xF0, 0x7E, 
0x08, 0x1E, 0x00, 0x43, 0x02, 0x60, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x80, 0x60, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x80, 0x20, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x80, 0x60, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x80, 0x60, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x20, 
0x04, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x2A, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x7F, 0x80, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 
0x00, 0x00, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x7F, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x00, 
0x00, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x35, 0x80, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 
0x00, 0x00, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x60, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0xC0, 0x30, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x01, 
0x80, 0x18, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x00, 
0x00, 0x00, 0x70, 0x00, 0x00, 0x00, 0x00, 0x01, 
0x39, 0xC8, 0x00, 0x00, 0x00, 0x0B, 0x00, 0x00, 
0x00, 0x00, 0xD0, 0x00, 0x00, 0x00, 0x00, 0x02, 
0x29, 0x48, 0x00, 0x00, 0x00, 0x19, 0x00, 0x00, 
0x00, 0x00, 0x98, 0x00, 0x00, 0x00, 0x00, 0x02, 
0x39, 0xC8, 0x00, 0x00, 0x00, 0x13, 0x00, 0x00, 
0x00, 0x00, 0xC8, 0x00, 0x00, 0x00, 0x00, 0x04, 
0x39, 0xC8, 0x00, 0x00, 0x00, 0x1F, 0x80, 0x00, 
0x00, 0x01, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x04, 
0x00, 0x08, 0x00, 0x00, 0x00, 0x3F, 0x80, 0x00, 
0x00, 0x01, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x04, 
0x00, 0x08, 0x00, 0x00, 0x00, 0x3F, 0xC0, 0x00, 
0x00, 0x03, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x04, 
0x00, 0x08, 0x00, 0x00, 0x1F, 0xFF, 0xFF, 0xE0, 
0x07, 0xFF, 0xFF, 0xF8, 0x00, 0x00, 0x00, 0x06, 
0x00, 0x10, 0x00, 0x00, 0x1F, 0xFF, 0xFF, 0xE0, 
0x07, 0xFF, 0xFF, 0xF8, 0x00, 0x00, 0x00, 0x03, 
0x80, 0x30, 0x00, 0x00, 0x0F, 0xFF, 0xFF, 0x00, 
0x00, 0xFF, 0xFF, 0xF0, 0x00, 0x00, 0x00, 0x00, 
0xFF, 0xE0, 0x00, 0x00, 0x01, 0x3F, 0xF4, 0x00, 
0x00, 0x2F, 0xFC, 0x80, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x03, 0x89, 0x47, 0x00, 
0x00, 0xE2, 0x91, 0xC0, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x03, 0x80, 0x07, 0x00, 
0x00, 0xE0, 0x01, 0xC0, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x03, 0xF8, 0x7F, 0x00, 
0x00, 0xFE, 0x1F, 0xC0, 0x00, 0x0F, 0xE0, 0x00, 
0x30, 0x00, 0x00, 0x00, 0x02, 0xFC, 0xFD, 0x00, 
0x00, 0xBF, 0x3F, 0x40, 0x00, 0x1F, 0xF0, 0x00, 
0x30, 0x00, 0x00, 0x00, 0x06, 0xFB, 0x7D, 0x80, 
0x01, 0xBE, 0xDF, 0x60, 0x00, 0x38, 0x38, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0xD8, 0x6C, 0x00, 
0x00, 0x36, 0x1B, 0x00, 0x00, 0x70, 0x18, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0xD8, 0x6C, 0x00, 
0x00, 0x36, 0x1B, 0x00, 0x00, 0x60, 0x19, 0x9C, 
0x30, 0xF8, 0xCE, 0x00, 0x00, 0xD8, 0x6C, 0x00, 
0x00, 0x36, 0x1B, 0x00, 0x00, 0xC0, 0x09, 0xBE, 
0x31, 0xF8, 0x7F, 0x00, 0x00, 0xFF, 0xFE, 0x00, 
0x00, 0x7F, 0xFF, 0x00, 0x00, 0xC0, 0x19, 0xF6, 
0x31, 0x8C, 0xFB, 0x00, 0x3D, 0x80, 0x07, 0x00, 
0x00, 0xE0, 0x01, 0xBC, 0x00, 0xC0, 0x19, 0xC6, 
0x63, 0x0C, 0x63, 0x00, 0x6F, 0x00, 0x03, 0x80, 
0x01, 0xC0, 0x00, 0xF6, 0x00, 0xC0, 0x19, 0xC6, 
0x33, 0x0C, 0xE1, 0x00, 0xFE, 0x00, 0x01, 0x80, 
0x01, 0x80, 0x00, 0x7F, 0x00, 0xC0, 0x31, 0x86, 
0x23, 0x04, 0x63, 0x00, 0x7E, 0x00, 0x00, 0xC0, 
0x03, 0x00, 0x00, 0x7E, 0x00, 0x60, 0x31, 0x86, 
0x33, 0x0C, 0xC3, 0x00, 0x03, 0xC0, 0x00, 0x40, 
0x02, 0x00, 0x03, 0xC0, 0x00, 0x70, 0xE1, 0x86, 
0x63, 0x9C, 0xC1, 0x80, 0x00, 0x7F, 0xFF, 0xC0, 
0x03, 0xFF, 0xFE, 0x00, 0x00, 0x3F, 0xE1, 0x82, 
0x31, 0xF8, 0x43, 0x00, 0x00, 0x78, 0x01, 0xE0, 
0x07, 0x80, 0x1E, 0x00, 0x00, 0x1F, 0x80, 0x82, 
0x20, 0x60, 0x41, 0x80, 0x00, 0xF0, 0x00, 0xE0, 
0x07, 0x00, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0xE0, 0x00, 0x60, 
0x06, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

const unsigned char PROGMEM hex0_bmp[] =
{0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0x80, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 
0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x54, 0x00, 0x02, 0x00, 0x01, 
0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 
0x00, 0x28, 0x00, 0x07, 0x00, 0x01, 0x80, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x10, 
0x00, 0x0D, 0x80, 0x01, 0x80, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x06, 0x00, 0x10, 0x00, 0x0D, 
0x80, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x07, 0xE0, 0x7F, 0x83, 0xFF, 0x87, 0xC3, 
0x80, 0x15, 0x50, 0x00, 0xFC, 0x00, 0x00, 0x05, 
0xF1, 0xFF, 0xF1, 0xFF, 0xF7, 0xFF, 0x80, 0x0F, 
0xE4, 0x03, 0x03, 0x00, 0x00, 0x07, 0xF9, 0xFF, 
0xF1, 0xFF, 0xFF, 0xFE, 0x80, 0xB0, 0x18, 0x0C, 
0x00, 0xC0, 0x00, 0x06, 0xFF, 0xFF, 0xF0, 0x7F, 
0xFF, 0xFF, 0x80, 0x40, 0x05, 0x10, 0x00, 0x20, 
0x00, 0x07, 0xFF, 0xFF, 0xFA, 0x3F, 0xFF, 0xFD, 
0x82, 0x80, 0x02, 0x10, 0x00, 0x20, 0x00, 0x05, 
0xFF, 0xFF, 0xFE, 0x0F, 0xFF, 0xFF, 0x81, 0x00, 
0x01, 0x60, 0x00, 0x10, 0x00, 0x07, 0xFF, 0xFF, 
0xFF, 0x23, 0xFF, 0xFE, 0x86, 0x00, 0x00, 0xA0, 
0x30, 0x10, 0x00, 0x06, 0xFF, 0xFF, 0xFF, 0xE1, 
0xFF, 0xFF, 0x82, 0x00, 0x00, 0xA0, 0x78, 0x10, 
0x00, 0x07, 0xFF, 0xFF, 0xFF, 0xF8, 0xFF, 0xFD, 
0x84, 0x03, 0x80, 0x60, 0x78, 0x10, 0x00, 0x00, 
0x1F, 0xFF, 0xFF, 0xF8, 0x00, 0xFF, 0x8C, 0x07, 
0xC0, 0x60, 0x30, 0x10, 0x00, 0x00, 0x07, 0xFF, 
0xFF, 0xFE, 0x14, 0x7E, 0x84, 0x07, 0xC0, 0x50, 
0x00, 0x20, 0x00, 0x09, 0x11, 0xFF, 0xFF, 0xFF, 
0x3C, 0x3F, 0x84, 0x07, 0xC0, 0x50, 0x00, 0x20, 
0x00, 0x1F, 0xF0, 0xFF, 0xFF, 0xFE, 0x3D, 0x1D, 
0x8C, 0x03, 0x80, 0x6C, 0x00, 0xC0, 0x01, 0x75, 
0xFA, 0x7F, 0xFF, 0xF8, 0x3F, 0x1F, 0x82, 0x00, 
0x00, 0x83, 0x03, 0x00, 0x01, 0xC7, 0xFE, 0x3F, 
0xFF, 0xC0, 0x7F, 0x3E, 0x86, 0x00, 0x00, 0x80, 
0xFC, 0x00, 0x17, 0x06, 0xFF, 0x3F, 0xFF, 0x87, 
0xFF, 0xFF, 0x81, 0x00, 0x01, 0x40, 0x00, 0x00, 
0x1C, 0x07, 0xFF, 0x40, 0x00, 0x0F, 0xFF, 0xFD, 
0x82, 0x80, 0x02, 0x00, 0x00, 0x01, 0x70, 0x05, 
0xFF, 0xC0, 0x00, 0x3F, 0xFF, 0xFF, 0x80, 0x40, 
0x05, 0x00, 0x00, 0x01, 0xC0, 0x07, 0xFF, 0xE1, 
0x11, 0x0F, 0xFF, 0xFE, 0x80, 0xB0, 0x18, 0x00, 
0x00, 0x17, 0x00, 0x06, 0xFF, 0xE7, 0xF1, 0xE1, 
0xFF, 0xFF, 0x80, 0x2F, 0xE4, 0x00, 0x48, 0x1C, 
0x00, 0x07, 0xFF, 0xE7, 0xF3, 0xFD, 0x7F, 0xFD, 
0x80, 0x09, 0x20, 0x02, 0xFD, 0x70, 0x00, 0x05, 
0xFF, 0x0F, 0xE3, 0xFF, 0x09, 0xFF, 0x80, 0x00, 
0x00, 0x01, 0x02, 0x00, 0x00, 0x07, 0xF8, 0x1F, 
0xE3, 0xFF, 0xFC, 0xFE, 0x83, 0xC0, 0x00, 0x02, 
0x01, 0x40, 0x00, 0x06, 0xF8, 0xFF, 0xD7, 0xFF, 
0xFE, 0x1F, 0x8C, 0x30, 0x00, 0x0C, 0x00, 0x80, 
0x00, 0x07, 0xFA, 0xFF, 0x9F, 0xFF, 0xFF, 0xCD, 
0x88, 0x10, 0x00, 0x04, 0x30, 0x88, 0x20, 0x05, 
0xFC, 0xFF, 0x9F, 0xFF, 0xFF, 0xC7, 0x90, 0x08, 
0x00, 0x0C, 0x30, 0xCF, 0xE0, 0x07, 0xFF, 0xFE, 
0x1F, 0xFF, 0xFF, 0xC6, 0x91, 0x8A, 0xA0, 0x04, 
0x00, 0x88, 0x20, 0x06, 0xFF, 0xFC, 0x3F, 0xFF, 
0xFF, 0xCF, 0x91, 0x89, 0xC0, 0x0A, 0x01, 0x0F, 
0xE0, 0x07, 0xFF, 0xFC, 0x3F, 0xFF, 0xFF, 0x9D, 
0x90, 0x0A, 0x28, 0x01, 0x02, 0x88, 0x20, 0x05, 
0xFF, 0xF9, 0x0F, 0xFF, 0xFF, 0x1F, 0x88, 0x1C, 
0x10, 0x00, 0xFC, 0x0F, 0xE0, 0x07, 0xFF, 0xF3, 
0x83, 0xFF, 0xFF, 0x3E, 0x8C, 0x34, 0x98, 0x7E, 
0x52, 0x0C, 0x10, 0x06, 0xFF, 0xE7, 0xE1, 0xFF, 
0xFE, 0x7F, 0x83, 0xC4, 0x11, 0x81, 0x80, 0x07, 
0xF0, 0x07, 0xFF, 0xCF, 0xF4, 0x7F, 0xFE, 0x7D, 
0x80, 0x0A, 0x36, 0x00, 0x60, 0x04, 0x10, 0x05, 
0xF1, 0x1F, 0xFE, 0x0F, 0xE0, 0xFF, 0x80, 0x01, 
0xC8, 0x00, 0x10, 0x07, 0xF0, 0x07, 0xF0, 0x7F, 
0xFF, 0x03, 0xC0, 0xFE, 0x80, 0x02, 0xA8, 0x00, 
0x10, 0x04, 0x10, 0x06, 0xE0, 0x7F, 0xFF, 0xC0, 
0x00, 0x7F, 0x80, 0x00, 0x10, 0x00, 0x08, 0x07, 
0xF0, 0x07, 0xE0, 0x3F, 0xFF, 0xF8, 0x00, 0x7D, 
0x80, 0x00, 0x10, 0x1C, 0x08, 0x04, 0x10, 0x05, 
0xE0, 0x3F, 0xFF, 0xFC, 0x00, 0x7F, 0x80, 0x00, 
0x10, 0x3E, 0x08, 0x07, 0xF0, 0x07, 0xE0, 0x3F, 
0xFF, 0xFE, 0x00, 0x7E, 0x80, 0x00, 0x10, 0x3E, 
0x08, 0x04, 0x10, 0x06, 0xE8, 0xBF, 0xFF, 0xFE, 
0x29, 0x7F, 0x80, 0x00, 0x10, 0x3E, 0x08, 0x07, 
0xF0, 0x07, 0xF2, 0x7F, 0xFF, 0xFF, 0x50, 0xFD, 
0x80, 0x00, 0x10, 0x1C, 0x08, 0x04, 0x10, 0x05, 
0xFF, 0xFF, 0x8F, 0xFF, 0x84, 0xFF, 0x80, 0x00, 
0x08, 0x00, 0x10, 0x07, 0xF0, 0x07, 0xFF, 0xFF, 
0x07, 0x1F, 0xEF, 0xFE, 0x80, 0x00, 0x08, 0x00, 
0x10, 0x04, 0x10, 0x06, 0xFF, 0xFF, 0x10, 0x0F, 
0xCF, 0xFF, 0x80, 0x00, 0x06, 0x00, 0x60, 0x07, 
0xF0, 0x07, 0xFF, 0xFF, 0x98, 0x00, 0x1F, 0xFD, 
0x80, 0x00, 0x01, 0x81, 0x80, 0x04, 0x10, 0x05, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x80, 0x00, 
0x00, 0x7E, 0x00, 0x07, 0xF0, 0x07, 0xBB, 0xBB, 
0xBB, 0xBB, 0xBB, 0xBB, 0x80, 0x00, 0x00, 0x00, 
0x00, 0x04, 0x10, 0x06, 0xEE, 0xEE, 0xEE, 0xEE, 
0xEE, 0xEF, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE};

const unsigned char PROGMEM hex1_bmp[] = 
{0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0x80, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 
0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x54, 0x00, 0x02, 0x00, 0x01, 
0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 
0x00, 0x28, 0x00, 0x07, 0x00, 0x01, 0x80, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x10, 
0x00, 0x0D, 0x80, 0x01, 0x80, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x06, 0x00, 0x10, 0x00, 0x0D, 
0x80, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x07, 0xE0, 0x7F, 0x83, 0xFF, 0x87, 0xC3, 
0x80, 0x0A, 0xA0, 0x00, 0xFC, 0x00, 0x00, 0x05, 
0xF1, 0xFF, 0xF1, 0xFF, 0xF7, 0xFF, 0x80, 0x0F, 
0xE8, 0x03, 0x03, 0x00, 0x00, 0x07, 0xF9, 0xFF, 
0xF1, 0xFF, 0xFF, 0xFE, 0x80, 0xB0, 0x1A, 0x0C, 
0x00, 0xC0, 0x00, 0x06, 0xFF, 0xFF, 0xF8, 0x7F, 
0xFF, 0xFF, 0x80, 0x40, 0x04, 0x10, 0x00, 0x20, 
0x00, 0x07, 0xFF, 0xFF, 0xF8, 0x3F, 0xFF, 0xFD, 
0x82, 0x80, 0x02, 0x90, 0x00, 0x20, 0x00, 0x05, 
0xFF, 0xFF, 0xFE, 0x0F, 0xFF, 0xFF, 0x81, 0x00, 
0x01, 0x20, 0x00, 0x10, 0x00, 0x07, 0xFF, 0xFF, 
0xFF, 0x23, 0xFF, 0xFE, 0x86, 0x00, 0x00, 0xA0, 
0x30, 0x10, 0x00, 0x06, 0xFF, 0xFF, 0xFF, 0xE9, 
0xFF, 0xFF, 0x82, 0x00, 0x00, 0xE0, 0x78, 0x10, 
0x00, 0x07, 0xFF, 0xFF, 0xFF, 0xF8, 0xFF, 0xFD, 
0x8C, 0x03, 0x80, 0x60, 0x78, 0x10, 0x00, 0x00, 
0x1F, 0xFF, 0xFF, 0xF8, 0x00, 0xFF, 0x84, 0x07, 
0xC0, 0x60, 0x30, 0x10, 0x00, 0x00, 0x07, 0xFF, 
0xFF, 0xFE, 0x10, 0x7E, 0x84, 0x07, 0xC0, 0x50, 
0x00, 0x20, 0x00, 0x02, 0x41, 0xFF, 0xFF, 0xFF, 
0x3C, 0x3F, 0x8C, 0x07, 0xC0, 0x70, 0x00, 0x20, 
0x00, 0x5F, 0xF4, 0xFF, 0xFF, 0xFE, 0x3D, 0x1D, 
0x84, 0x03, 0x80, 0x4C, 0x00, 0xC0, 0x00, 0x75, 
0xF8, 0x7F, 0xFF, 0xF8, 0x3F, 0x1F, 0x82, 0x00, 
0x00, 0x83, 0x03, 0x00, 0x05, 0xC7, 0xFE, 0xBF, 
0xFF, 0xC0, 0x7F, 0x3E, 0x82, 0x00, 0x00, 0xC0, 
0xFC, 0x00, 0x07, 0x06, 0xFF, 0x3F, 0xFF, 0x87, 
0xFF, 0xFF, 0x85, 0x00, 0x01, 0x00, 0x00, 0x00, 
0x5C, 0x07, 0xFF, 0x00, 0x00, 0x0F, 0xFF, 0xFD, 
0x80, 0x80, 0x02, 0x80, 0x00, 0x00, 0x70, 0x05, 
0xFF, 0xC0, 0x00, 0x3F, 0xFF, 0xFF, 0x81, 0x40, 
0x04, 0x00, 0x00, 0x05, 0xC0, 0x07, 0xFF, 0xE0, 
0x50, 0x0F, 0xFF, 0xFE, 0x80, 0x30, 0x1A, 0x00, 
0x00, 0x07, 0x00, 0x06, 0xFF, 0xE7, 0xF5, 0xE5, 
0xFF, 0xFF, 0x80, 0x4F, 0xF0, 0x01, 0x24, 0x5C, 
0x00, 0x07, 0xFF, 0xF7, 0xF3, 0xFC, 0x7F, 0xFD, 
0x80, 0x12, 0x48, 0x00, 0xFC, 0x70, 0x00, 0x05, 
0xFF, 0x0F, 0xE3, 0xFF, 0x21, 0xFF, 0x80, 0x00, 
0x00, 0x01, 0x02, 0x80, 0x00, 0x07, 0xF8, 0x5F, 
0xE3, 0xFF, 0xFC, 0xFE, 0x83, 0xC0, 0x00, 0x0A, 
0x01, 0x00, 0x00, 0x06, 0xF8, 0xFF, 0xD7, 0xFF, 
0xFE, 0x1F, 0x8C, 0x30, 0x00, 0x04, 0x00, 0xC0, 
0x00, 0x07, 0xF8, 0xFF, 0x9F, 0xFF, 0xFF, 0xCD, 
0x88, 0x10, 0x00, 0x0C, 0x30, 0x88, 0x20, 0x05, 
0xFC, 0xFF, 0x9F, 0xFF, 0xFF, 0xC7, 0x90, 0x08, 
0x00, 0x04, 0x30, 0x8F, 0xE0, 0x07, 0xFF, 0xFE, 
0x1F, 0xFF, 0xFF, 0xC6, 0x91, 0x89, 0x40, 0x0C, 
0x00, 0xC8, 0x20, 0x06, 0xFF, 0xFC, 0x3F, 0xFF, 
0xFF, 0xCF, 0x91, 0x8D, 0xD0, 0x02, 0x01, 0x0F, 
0xE0, 0x07, 0xFF, 0xFC, 0x3F, 0xFF, 0xFF, 0x9D, 
0x90, 0x0A, 0x20, 0x05, 0x03, 0x08, 0x20, 0x05, 
0xFF, 0xF9, 0x0F, 0xFF, 0xFF, 0x1F, 0x88, 0x1C, 
0x18, 0x00, 0xFC, 0x8F, 0xE0, 0x07, 0xFF, 0xF3, 
0x83, 0xFF, 0xFF, 0x3E, 0x8C, 0x3C, 0x90, 0x7E, 
0xA4, 0x0C, 0x10, 0x06, 0xFF, 0xE7, 0xE1, 0xFF, 
0xFE, 0x7F, 0x83, 0xC4, 0x19, 0x81, 0x80, 0x07, 
0xF0, 0x07, 0xFF, 0xCF, 0xF4, 0x7F, 0xFE, 0x7D, 
0x80, 0x06, 0x36, 0x00, 0x60, 0x04, 0x10, 0x05, 
0xF1, 0x1F, 0xFE, 0x0F, 0xE0, 0xFF, 0x80, 0x05, 
0xD8, 0x00, 0x10, 0x07, 0xF0, 0x07, 0xF0, 0x7F, 
0xFF, 0x03, 0xC0, 0xFE, 0x80, 0x01, 0x48, 0x00, 
0x10, 0x04, 0x10, 0x06, 0xE0, 0x7F, 0xFF, 0xC0, 
0x00, 0x7F, 0x80, 0x00, 0x10, 0x00, 0x08, 0x07, 
0xF0, 0x07, 0xE0, 0x3F, 0xFF, 0xF8, 0x00, 0x7D, 
0x80, 0x00, 0x10, 0x1C, 0x08, 0x04, 0x10, 0x05, 
0xE0, 0x3F, 0xFF, 0xFC, 0x00, 0x7F, 0x80, 0x00, 
0x10, 0x3E, 0x08, 0x07, 0xF0, 0x07, 0xF0, 0x3F, 
0xFF, 0xFE, 0x00, 0x7E, 0x80, 0x00, 0x10, 0x3E, 
0x08, 0x04, 0x10, 0x06, 0xE0, 0x3F, 0xFF, 0xFE, 
0x18, 0xFF, 0x80, 0x00, 0x10, 0x3E, 0x08, 0x07, 
0xF0, 0x07, 0xF5, 0x7F, 0xFF, 0xFF, 0x0C, 0xFD, 
0x80, 0x00, 0x10, 0x1C, 0x08, 0x04, 0x10, 0x05, 
0xFF, 0xFF, 0x8F, 0xFF, 0x85, 0xFF, 0x80, 0x00, 
0x08, 0x00, 0x10, 0x07, 0xF0, 0x07, 0xFF, 0xFF, 
0x07, 0x1F, 0xEF, 0xFE, 0x80, 0x00, 0x08, 0x00, 
0x10, 0x04, 0x10, 0x06, 0xFF, 0xFF, 0x40, 0x0F, 
0xCF, 0xFF, 0x80, 0x00, 0x06, 0x00, 0x60, 0x07, 
0xF0, 0x07, 0xFF, 0xFF, 0xA8, 0x00, 0x1F, 0xFD, 
0x80, 0x00, 0x01, 0x81, 0x80, 0x04, 0x10, 0x05, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x80, 0x00, 
0x00, 0x7E, 0x00, 0x07, 0xF0, 0x07, 0xBB, 0xBB, 
0xBB, 0xBB, 0xBB, 0xBB, 0x80, 0x00, 0x00, 0x00, 
0x00, 0x04, 0x10, 0x06, 0xEE, 0xEE, 0xEE, 0xEE, 
0xEE, 0xEF, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE};


const unsigned char PROGMEM troll_bmp[] =
{0x00, 0x00, 0xF0, 0x00, 0x00, 0x0F, 0x10, 0x00, 
0x00, 0x08, 0x10, 0x00, 0x00, 0x30, 0xD8, 0x00, 
0x00, 0x26, 0xC8, 0x00, 0x00, 0x26, 0x08, 0x00, 
0x00, 0x23, 0x88, 0x00, 0x00, 0x20, 0x10, 0x00, 
0x00, 0x20, 0xFC, 0x00, 0x00, 0x17, 0x8E, 0x00, 
0x00, 0x3C, 0x0F, 0x00, 0x00, 0x30, 0x0B, 0x80, 
0x00, 0x70, 0x09, 0xC0, 0x00, 0xB0, 0x09, 0xC0, 
0x01, 0xB0, 0xE8, 0xC0, 0x01, 0x7F, 0xB8, 0xE0, 
0x01, 0x48, 0x0C, 0xE0, 0x01, 0x98, 0x04, 0x70, 
0x01, 0x90, 0x04, 0x6C, 0x07, 0xDC, 0x0C, 0xC4, 
0x07, 0x47, 0x78, 0x84, 0x04, 0x4E, 0xE8, 0x84, 
0x04, 0xDA, 0x6C, 0x8C, 0x07, 0x92, 0x24, 0xF8, 
0x00, 0x12, 0x36, 0x00, 0x00, 0x12, 0x12, 0x00, 
0x00, 0x12, 0x3A, 0x00, 0x00, 0x1C, 0x3E, 0x00, 
0x00, 0x3C, 0x36, 0x00, 0x00, 0x2C, 0x22, 0x00, 
0x00, 0x44, 0x22, 0x00, 0x00, 0x7C, 0x3C, 0x00};

const unsigned char PROGMEM thief_bmp[] = 
{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xF8, 0x3F, 0xFF, 0xFF, 0xF0, 0x1F, 0xFF, 
0xFF, 0xF0, 0x1F, 0xFF, 0xFF, 0xF3, 0x9F, 0xFF, 
0xFF, 0xF5, 0x5F, 0xFF, 0xFF, 0xF7, 0xDF, 0xFF, 
0xFF, 0xF5, 0x5F, 0xFF, 0xFF, 0xF2, 0x9F, 0xFF, 
0xFF, 0xFB, 0xBF, 0xFF, 0xFF, 0xFC, 0x7F, 0xFF, 
0xFF, 0xF8, 0x3F, 0xFF, 0xFF, 0xE3, 0x8F, 0xFF, 
0xFF, 0xE0, 0x0F, 0xFF, 0xFF, 0xEB, 0xAF, 0xFF, 
0xFF, 0xE8, 0x2F, 0xD3, 0xFF, 0xEB, 0xAF, 0xC7, 
0xFF, 0xC8, 0x27, 0xE7, 0xFF, 0xEB, 0xAF, 0xC7, 
0xFF, 0xF8, 0x3F, 0x07, 0xFF, 0xF9, 0x3F, 0x73, 
0xFF, 0xF9, 0x3E, 0xFB, 0xFF, 0xF9, 0x3C, 0xD9, 
0xFF, 0xF1, 0x1D, 0xAD, 0xFF, 0xF3, 0x9E, 0xDD, 
0xFF, 0xC3, 0x86, 0xF9, 0xFF, 0xC3, 0x87, 0x03};

const unsigned char PROGMEM dwarf_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x00, 
0x00, 0x00, 0x7C, 0x00, 0x00, 0x00, 0xCC, 0x00, 
0x00, 0x01, 0x88, 0x00, 0x00, 0x03, 0x08, 0x00, 
0x00, 0x06, 0x08, 0x00, 0x00, 0x0C, 0x08, 0x00, 
0x00, 0x18, 0x08, 0x00, 0x00, 0x1F, 0xF8, 0x00, 
0x00, 0x18, 0x08, 0x00, 0x00, 0x10, 0x08, 0x00, 
0x00, 0x12, 0x48, 0x00, 0x00, 0x10, 0x08, 0x00, 
0x00, 0x13, 0xF8, 0x00, 0x00, 0x1C, 0x10, 0x00, 
0x00, 0x0D, 0xD0, 0x00, 0x00, 0x0C, 0x14, 0x00, 
0x00, 0x3E, 0x3E, 0x00, 0x00, 0x7A, 0x2F, 0x00, 
0x00, 0xD2, 0x6D, 0x80, 0x00, 0x93, 0xC5, 0xC0, 
0x01, 0x91, 0x84, 0xC0, 0x01, 0xB0, 0x86, 0xC0, 
0x00, 0x20, 0x02, 0x00, 0x00, 0x20, 0x02, 0x00, 
0x00, 0x3F, 0xFE, 0x00, 0x00, 0x06, 0x30, 0x00, 
0x00, 0x06, 0x30, 0x00, 0x00, 0x3E, 0x38, 0x00, 
0x00, 0x7E, 0x3E, 0x00, 0x00, 0x7E, 0x3F, 0x00};

const unsigned char PROGMEM goblin_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 
0x00, 0x82, 0x04, 0x00, 0x00, 0x44, 0x08, 0x00, 
0x00, 0x24, 0x10, 0x00, 0x00, 0x22, 0x08, 0x00, 
0x00, 0x41, 0x08, 0x00, 0x00, 0x81, 0x10, 0x00, 
0x00, 0x82, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 
0x00, 0x07, 0xFC, 0x00, 0x00, 0x18, 0x02, 0x00, 
0x00, 0x20, 0x01, 0x00, 0x00, 0x4E, 0x38, 0x80, 
0x00, 0x91, 0x44, 0x80, 0x00, 0x91, 0x44, 0x80, 
0x00, 0x9F, 0x7C, 0x80, 0x00, 0x8E, 0x39, 0x80, 
0x01, 0x80, 0x01, 0x80, 0x03, 0x8F, 0xF1, 0xC0, 
0x02, 0x84, 0x23, 0x40, 0x02, 0xC3, 0xC6, 0x40, 
0x02, 0x60, 0x0C, 0x40, 0x02, 0x3C, 0x30, 0x40, 
0x02, 0x0F, 0xE0, 0x40, 0x02, 0x08, 0x40, 0xA0, 
0x05, 0x08, 0x40, 0x00, 0x00, 0x08, 0x60, 0x00, 
0x00, 0x08, 0x20, 0x00, 0x00, 0x3C, 0x38, 0x00};

const unsigned char PROGMEM golem_bmp[] = 
{0x00, 0x03, 0xC0, 0x00, 0x00, 0x07, 0xC0, 0x00, 
0x00, 0x0D, 0x40, 0x00, 0x00, 0x0E, 0xE0, 0x00, 
0x00, 0x0F, 0xE0, 0x00, 0x00, 0x0C, 0x60, 0x00, 
0x00, 0x0F, 0xE0, 0x00, 0x00, 0x0F, 0xC0, 0x00, 
0x00, 0x3F, 0xF8, 0x00, 0x00, 0x7F, 0xFC, 0x00, 
0x00, 0xFF, 0xFE, 0x00, 0x00, 0xFC, 0x7E, 0x00, 
0x00, 0xDB, 0xEE, 0x00, 0x01, 0xDB, 0xEF, 0x00, 
0x01, 0xDA, 0x2F, 0x00, 0x01, 0x9B, 0xA7, 0x00, 
0x01, 0x9C, 0x67, 0x00, 0x01, 0x9F, 0xE7, 0x00, 
0x01, 0xDF, 0xEF, 0x00, 0x01, 0xDF, 0xEF, 0x00, 
0x00, 0xDF, 0xEC, 0x00, 0x00, 0xDF, 0xEC, 0x00, 
0x00, 0xDC, 0xEC, 0x00, 0x00, 0x1C, 0xE0, 0x00, 
0x00, 0x1C, 0xE0, 0x00, 0x00, 0x1C, 0xE0, 0x00, 
0x00, 0x1C, 0xE0, 0x00, 0x00, 0x1C, 0xE0, 0x00, 
0x00, 0x1C, 0xE0, 0x00, 0x00, 0x1C, 0xE0, 0x00, 
0x00, 0xFC, 0xFC, 0x00, 0x00, 0xFC, 0xFC, 0x00};

const unsigned char PROGMEM wizard_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x00, 
0x00, 0x00, 0x1C, 0x00, 0x00, 0x00, 0x34, 0x00, 
0x00, 0x00, 0x24, 0x00, 0x20, 0x00, 0xCC, 0x00, 
0x10, 0x1D, 0x88, 0x00, 0x08, 0x1F, 0xC8, 0x00, 
0x08, 0x07, 0xFC, 0x00, 0x04, 0x03, 0x0F, 0x00, 
0x02, 0x02, 0x49, 0x00, 0x03, 0x0E, 0x08, 0x00, 
0x07, 0x83, 0x98, 0x00, 0x07, 0xC1, 0xF0, 0x00, 
0x03, 0x63, 0xD0, 0x00, 0x02, 0x37, 0x88, 0x00, 
0x03, 0x1D, 0x87, 0x00, 0x01, 0x81, 0x81, 0x80, 
0x00, 0x80, 0x00, 0xC0, 0x00, 0xC0, 0x00, 0x40, 
0x00, 0x58, 0x02, 0x40, 0x00, 0x70, 0x02, 0x40, 
0x00, 0x30, 0x02, 0x40, 0x00, 0x20, 0x03, 0xC0, 
0x00, 0x20, 0x07, 0x80, 0x00, 0x20, 0x03, 0x80, 
0x00, 0x20, 0x01, 0x00, 0x00, 0x60, 0x01, 0x00, 
0x00, 0x60, 0x01, 0x00, 0x00, 0x7F, 0xFE, 0x00, 
0x00, 0x02, 0x40, 0x00, 0x00, 0x0E, 0x70, 0x00};

const unsigned char PROGMEM street_urchin_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xE0, 0x00, 
0x00, 0x07, 0xF0, 0x00, 0x00, 0x01, 0xF0, 0x00, 
0x00, 0x06, 0x20, 0x00, 0x00, 0x0D, 0x20, 0x00, 
0x00, 0x18, 0x20, 0x00, 0x00, 0x0C, 0x60, 0x00, 
0x00, 0x06, 0x40, 0x00, 0x00, 0x03, 0x80, 0x00, 
0x00, 0x01, 0x80, 0x00, 0x00, 0x03, 0xC0, 0x00, 
0x00, 0x1C, 0x38, 0x00, 0x00, 0x10, 0x18, 0x00, 
0x00, 0x18, 0x30, 0x00, 0x00, 0x28, 0x28, 0x00, 
0x00, 0x2F, 0xE8, 0x00, 0x00, 0x28, 0x28, 0x00, 
0x00, 0x09, 0x20, 0x00, 0x00, 0x0E, 0xE0, 0x00, 
0x00, 0x04, 0x40, 0x00, 0x00, 0x1C, 0x70, 0x00};

const unsigned char PROGMEM gnome_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0xF0, 0x00, 
0x00, 0x01, 0xFC, 0x00, 0x00, 0x03, 0xCC, 0x00, 
0x00, 0x07, 0xC0, 0x00, 0x00, 0x0A, 0xA0, 0x00, 
0x00, 0x08, 0x40, 0x00, 0x00, 0x04, 0x80, 0x00, 
0x00, 0x3F, 0xE0, 0x00, 0x00, 0x60, 0x30, 0x00, 
0x00, 0x50, 0x50, 0x00, 0x00, 0x4F, 0x90, 0x00, 
0x00, 0x68, 0xB0, 0x00, 0x00, 0x0F, 0x80, 0x00, 
0x00, 0x18, 0xC0, 0x00, 0x00, 0x38, 0xE0, 0x00};

const unsigned char PROGMEM librarian_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x06, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 
0x10, 0x80, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 
0x40, 0x20, 0x00, 0x00, 0x20, 0x10, 0x00, 0x00, 
0x10, 0x10, 0x00, 0x00, 0x2C, 0x20, 0x00, 0x00, 
0x3C, 0x40, 0x00, 0x00, 0x3A, 0x80, 0x00, 0x00, 
0x19, 0x00, 0x00, 0x00, 0x18, 0x03, 0xE0, 0x00, 
0x0C, 0x0C, 0x10, 0x00, 0x0C, 0x08, 0x10, 0x00, 
0x0E, 0x0A, 0x90, 0x00, 0x07, 0x08, 0x10, 0x00, 
0x07, 0xCB, 0x90, 0x00, 0x01, 0xFC, 0x50, 0x00, 
0x00, 0xF8, 0x3F, 0x80, 0x00, 0x6B, 0xAF, 0xC0, 
0x00, 0x44, 0x2F, 0xE0, 0x00, 0x47, 0xC6, 0x70, 
0x00, 0x40, 0x02, 0x30, 0x00, 0x40, 0x06, 0x30, 
0x00, 0x60, 0x04, 0x38, 0x00, 0x20, 0x04, 0x18, 
0x00, 0x3F, 0xFC, 0x18, 0x00, 0x18, 0x18, 0x18, 
0x00, 0xF8, 0x1F, 0x30, 0x01, 0xF0, 0x0F, 0x9C};

const unsigned char PROGMEM assassin_bmp[] = 
{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0x1F, 0xFF, 0xFF, 0xFE, 0x0F, 0xFF, 
0xFF, 0xFD, 0xCF, 0xFF, 0xFF, 0xFD, 0x47, 0xFF, 
0xFF, 0xF1, 0xC7, 0xFF, 0xFF, 0xEF, 0xC7, 0xFF, 
0xFF, 0xF2, 0xD7, 0xFF, 0xFF, 0xFD, 0x9F, 0xFF, 
0xFB, 0xFD, 0xBF, 0xFF, 0xFD, 0x78, 0x1F, 0xFF, 
0xFE, 0xF0, 0x0F, 0xFF, 0xFC, 0x60, 0x07, 0xFF, 
0xFF, 0x28, 0x27, 0xFF, 0xFF, 0x88, 0x27, 0xFF, 
0xFF, 0xF8, 0x27, 0xFF, 0xFF, 0xF8, 0x27, 0xFF, 
0xFF, 0xF8, 0x17, 0xFF, 0xFF, 0xFB, 0xAF, 0xFF, 
0xFF, 0xF8, 0x3F, 0xFF, 0xFF, 0xF8, 0x3F, 0xFF, 
0xFF, 0xF9, 0x3F, 0xFF, 0xFF, 0xF9, 0x3F, 0xFF, 
0xFF, 0xF9, 0x3F, 0xFF, 0xFF, 0xF1, 0x0F, 0xFF};

const unsigned char PROGMEM witch_bmp[] = 
{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF0, 0x3F, 
0xFF, 0xFF, 0xC6, 0x7F, 0xFF, 0xFF, 0x9C, 0xFF, 
0xFF, 0xFF, 0x39, 0xFF, 0xFF, 0xFE, 0x73, 0xFF, 
0xFF, 0xF0, 0x77, 0xFF, 0xFE, 0xFE, 0x87, 0xFF, 
0xFE, 0xFD, 0xE8, 0x7F, 0xFE, 0xFA, 0x67, 0xFF, 
0xFE, 0xE7, 0xF3, 0xFF, 0xFC, 0xCF, 0xB3, 0xFF, 
0xFD, 0xF2, 0x61, 0xFF, 0xFD, 0xFD, 0xD8, 0xFF, 
0xFD, 0xF0, 0x0C, 0xFF, 0xFD, 0xE1, 0xC6, 0xFF, 
0xF8, 0x63, 0xC7, 0xFF, 0xF8, 0x0B, 0xD7, 0xFF, 
0xF9, 0x9B, 0xD7, 0xFF, 0xFB, 0xFB, 0xD3, 0xFF, 
0xF3, 0xF8, 0x13, 0xFF, 0xF3, 0xF7, 0xEF, 0xFF, 
0xF3, 0xEF, 0xF7, 0xFF, 0xE1, 0xC0, 0x03, 0xFF, 
0xD5, 0xF9, 0x9F, 0xFF, 0xCA, 0xD9, 0x9B, 0xFF, 
0xAA, 0xC1, 0x83, 0xFF, 0xAA, 0xC1, 0x83, 0xFF};

const unsigned char PROGMEM werewolf_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0x00, 
0x00, 0x01, 0x20, 0x00, 0x00, 0x02, 0x10, 0x00, 
0x00, 0x0A, 0x14, 0x00, 0x00, 0x07, 0xF8, 0x00, 
0x00, 0x02, 0x10, 0x00, 0x00, 0x06, 0xA8, 0x00, 
0x00, 0x06, 0x08, 0x00, 0x00, 0x0E, 0x28, 0x00, 
0x00, 0x0E, 0xCC, 0x00, 0x00, 0x0E, 0x0E, 0x00, 
0x00, 0x1D, 0x36, 0x00, 0x00, 0x13, 0xF7, 0x00, 
0x00, 0x37, 0x3B, 0x00, 0x00, 0x3F, 0x3B, 0x00, 
0x00, 0x3E, 0x1C, 0x00, 0x00, 0x2A, 0x16, 0x00, 
0x00, 0x1A, 0x16, 0x00, 0x00, 0x1B, 0xF2, 0xC0, 
0x00, 0x35, 0x53, 0x80, 0x00, 0x09, 0x4F, 0x00, 
0x00, 0x12, 0x54, 0x80, 0x00, 0x14, 0xA4, 0x40, 
0x00, 0x1F, 0xFC, 0x20, 0x00, 0x07, 0x60, 0x10, 
0x00, 0x06, 0x60, 0x00, 0x00, 0x06, 0x60, 0x00, 
0x00, 0x06, 0x60, 0x00, 0x00, 0x0E, 0x70, 0x00};

const unsigned char PROGMEM death_bmp[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x0F, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x10, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x60, 0x03, 0x80, 0x00, 0x00, 0x00, 0x00, 
0x00, 0xC0, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 
0x01, 0x80, 0x00, 0xF0, 0x00, 0x00, 0x00, 0x00, 
0x01, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 
0x06, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 
0x04, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 
0x08, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 
0x08, 0x0F, 0xFE, 0x00, 0xC0, 0x00, 0x00, 0x00, 
0x30, 0x7F, 0xFF, 0xE0, 0x40, 0x00, 0x00, 0x00, 
0x20, 0xFF, 0xFF, 0xE0, 0x40, 0x00, 0x00, 0x00, 
0x40, 0xFF, 0xFF, 0xF0, 0x60, 0x00, 0x00, 0x00, 
0x41, 0xFF, 0xFF, 0xF8, 0x20, 0x00, 0x00, 0x00, 
0x83, 0xFF, 0xFF, 0xFC, 0x30, 0x00, 0x00, 0x00, 
0x83, 0xFF, 0xFF, 0xFC, 0x10, 0x00, 0x00, 0x00, 
0x83, 0xE0, 0xF8, 0x3C, 0x10, 0x00, 0x00, 0x00, 
0x87, 0xFF, 0x07, 0xFC, 0x10, 0x00, 0x00, 0x00, 
0x87, 0xFF, 0xFF, 0xFE, 0x18, 0x00, 0x00, 0x00, 
0x87, 0xE1, 0xF8, 0x7F, 0x08, 0x00, 0x00, 0x00, 
0x87, 0xC0, 0xF0, 0x3F, 0x08, 0x00, 0x00, 0x00, 
0x87, 0x8C, 0xF3, 0x1F, 0x08, 0x00, 0x00, 0x00, 
0x87, 0x8C, 0xF3, 0x1F, 0x08, 0x00, 0x00, 0x00, 
0x87, 0x80, 0xF0, 0x1F, 0x08, 0x00, 0x00, 0x00, 
0x87, 0x81, 0xF8, 0x1F, 0x08, 0x00, 0x00, 0x00, 
0x87, 0xC3, 0xFC, 0x3F, 0x08, 0x00, 0x00, 0x00, 
0x87, 0xFF, 0xFF, 0xFF, 0x08, 0x01, 0x80, 0x00, 
0x87, 0xFF, 0xFF, 0xFF, 0x08, 0x01, 0xE0, 0x00, 
0x87, 0xFF, 0xFF, 0xFF, 0x08, 0x03, 0xB8, 0x00, 
0x87, 0xFF, 0xBF, 0xFF, 0x08, 0x03, 0x8E, 0x00, 
0x87, 0xFF, 0x1F, 0xFF, 0x08, 0x07, 0x03, 0x00, 
0x87, 0xFE, 0x0F, 0xFF, 0x08, 0x07, 0x01, 0x80, 
0x87, 0xFE, 0x0F, 0xFF, 0x08, 0x0E, 0xC0, 0x60, 
0x87, 0xFE, 0x0F, 0xFE, 0x08, 0x0E, 0x70, 0x20, 
0x87, 0xFE, 0xEF, 0xFE, 0x08, 0x1E, 0x18, 0x30, 
0x87, 0xDF, 0xFF, 0x7E, 0x08, 0x1C, 0x0C, 0x10, 
0x83, 0x9F, 0xFF, 0x3C, 0x18, 0x1C, 0x02, 0x08, 
0x83, 0x1F, 0xFF, 0x1C, 0x10, 0x38, 0x03, 0x08, 
0xC3, 0x01, 0xF0, 0x1C, 0x30, 0x38, 0x00, 0x88, 
0xC1, 0x0C, 0x06, 0x18, 0x20, 0x38, 0x00, 0x44, 
0xC1, 0x8D, 0xB6, 0x30, 0x20, 0x30, 0x00, 0x64, 
0xC1, 0xC1, 0xB0, 0x70, 0x60, 0x70, 0x00, 0x34, 
0x40, 0xEC, 0x06, 0xE0, 0x40, 0x70, 0x00, 0x12, 
0x40, 0x6D, 0xB6, 0xE0, 0xC0, 0x60, 0x00, 0x1A, 
0x40, 0x11, 0xB1, 0x80, 0x80, 0xE0, 0x00, 0x0E, 
0x40, 0x1C, 0x07, 0x01, 0x80, 0xE0, 0x00, 0x06, 
0x60, 0x07, 0xFE, 0x02, 0x00, 0xE0, 0x00, 0x06, 
0x3C, 0x00, 0xCC, 0x02, 0x00, 0xC0, 0x00, 0x02, 
0x06, 0x00, 0x78, 0x04, 0x00, 0xC0, 0x00, 0x02, 
0x03, 0x00, 0x48, 0x04, 0x01, 0xC0, 0x00, 0x00, 
0x01, 0xC0, 0x78, 0x04, 0x01, 0xC0, 0x00, 0x00, 
0x00, 0x60, 0xC4, 0x04, 0x01, 0xC0, 0x00, 0x00, 
0x00, 0x20, 0xFC, 0x04, 0x03, 0xC0, 0x00, 0x00, 
0x00, 0x20, 0x84, 0x0C, 0x03, 0xC0, 0x00, 0x00, 
0x00, 0x00, 0xFC, 0x08, 0x03, 0xC0, 0x00, 0x00};

const unsigned char PROGMEM cmot_bmp[] = 
{0x00, 0x00, 0x00, 0x00, 0xF8, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x07, 0xFC, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x07, 0xFC, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x0F, 0xFE, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x1F, 0xFE, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x2A, 0xAB, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x2A, 0xAB, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x2F, 0xFF, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x18, 0x07, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x18, 0x06, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x10, 0x03, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x23, 0x31, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x23, 0x31, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x20, 0x01, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x20, 0xC1, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x23, 0x83, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x22, 0x12, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x16, 0x26, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x17, 0xC4, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x19, 0xCF, 0xC0, 0x00, 0x00, 
0x00, 0x00, 0x01, 0xFC, 0x0B, 0x20, 0x00, 0x00, 
0x00, 0x00, 0x7E, 0xC6, 0x33, 0x1C, 0x00, 0x00, 
0x00, 0x03, 0xC0, 0xC3, 0xE3, 0x03, 0x00, 0x00, 
0x00, 0x1E, 0x00, 0xC0, 0x03, 0x01, 0x80, 0x00, 
0x00, 0x30, 0x00, 0xE0, 0x03, 0x00, 0xE0, 0x00, 
0x00, 0xC0, 0x00, 0xF8, 0x07, 0x00, 0x30, 0x00, 
0x01, 0x80, 0x00, 0xFC, 0x0F, 0x00, 0x0C, 0x00, 
0x03, 0x00, 0x00, 0xFF, 0x1F, 0x00, 0x04, 0x00, 
0x06, 0x00, 0x00, 0xFF, 0xFF, 0x30, 0x06, 0x00, 
0x04, 0x01, 0xF1, 0xFF, 0xFF, 0x1C, 0x03, 0x00, 
0x0C, 0x03, 0x11, 0xFF, 0xFF, 0x16, 0x01, 0x00, 
0x08, 0x02, 0x21, 0xFF, 0xFF, 0x19, 0x80, 0x80, 
0x08, 0x1C, 0x61, 0xFF, 0xFF, 0x88, 0xC0, 0xC0, 
0x10, 0x30, 0x41, 0xFF, 0xFF, 0x88, 0x40, 0x60, 
0x30, 0x40, 0x41, 0xFF, 0xFF, 0x8C, 0x60, 0x20, 
0x20, 0x80, 0x41, 0xFF, 0xFF, 0x84, 0x10, 0x20, 
0x61, 0x00, 0xC1, 0xF3, 0xFF, 0x86, 0x18, 0x20, 
0x42, 0x00, 0x83, 0xE5, 0xFF, 0x82, 0x06, 0x30, 
0x42, 0x01, 0x83, 0xED, 0xCF, 0x82, 0x02, 0x10, 
0x62, 0x01, 0x02, 0x2D, 0x24, 0x02, 0x02, 0x10, 
0x26, 0x03, 0x05, 0xC8, 0xF1, 0x02, 0x02, 0x10, 
0x24, 0x02, 0x04, 0x75, 0xC9, 0x82, 0x02, 0x10, 
0x24, 0x02, 0x1B, 0x93, 0xDE, 0xC2, 0x02, 0x10, 
0x24, 0x02, 0x24, 0x6D, 0x06, 0x32, 0x02, 0x10, 
0x3C, 0x02, 0x27, 0x9E, 0xFB, 0x8A, 0x03, 0x90, 
0x3C, 0x02, 0x1B, 0xED, 0xFD, 0x0A, 0x01, 0x90, 
0x7E, 0x06, 0x04, 0x6E, 0x02, 0xF2, 0x07, 0x90, 
0x7F, 0xFF, 0xFF, 0x8F, 0xFF, 0xFF, 0xFF, 0xB0, 
0x79, 0x80, 0x00, 0x00, 0x00, 0x00, 0x1F, 0xE0, 
0x38, 0x80, 0x00, 0x00, 0x00, 0x00, 0x37, 0xC0, 
0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x80, 
0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x00, 
0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 
0x0F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC, 0x00, 
0x00, 0x07, 0xFF, 0xFF, 0xFF, 0xFE, 0x00, 0x00};

//Define all of the suits as offsets into the spent_cards array
#define STAVES    0      //STAVES   = \ = 92
#define SWORDS    1      //SWORDS   = T = 84
#define CUPS      2      //CUPS     = Y = 89
#define COINS     3      //COINS    = o = 111
#define OCTOGRAMS 4      //OCTOGRAM = * = 42
#define ELEPHANTS 5      //ELEPHANT = J = 74
#define TERRAPINS 6      //TERRAPINS= & = 38  @=64
#define CROWNS    7      //CROWNS   = # = 35

//The suits are grouped into pairs:  Staves and swords (0), cups and coins (1), octograms and elephants (2), terrapins and crowns (3)
#define NUM_SUITS 8
//                  \  T  Y  o   *  J  &  #
byte suit_lut[8] = {92,84,89,111,42,74,38,35};

#define C_ACE    1
#define C_TWO    2
#define C_THREE  3
#define C_FOUR   4
#define C_FIVE   5
#define C_SIX    6
#define C_SEVEN  7
#define C_EIGHT  8
#define C_NINE   9
#define C_TEN    10
#define C_PRINCE 11
#define C_QUEEN  12
#define C_KING   13

//                   #   A   2   3   4   5   6   7   8   9   T   J   Q   K
byte card_lut[16] = {45, 65, 50, 51, 52, 53, 54, 55, 56, 57, 84, 74, 81, 75, 45,45 };  

//Track the game state FSM-style, so that we know what to do during our game loop
byte state;
byte substate;
#define STATE_START_SCREEN 0
#define STATE_MENU         1
#define STATE_CHARACTER    2
#define STATE_RULES        3
#define STATE_1ST_DRAW     5
#define STATE_BET          6
#define STATE_2ND_DRAW     7
#define STATE_EVAL         8
#define STATE_LOSE         9
#define STATE_DIE          10
#define STATE_FHI          11

byte selected_card;       //Draw this card outligned 
word tagged_cards;        //Draw these card highlighted

bool ignore_buttons;
byte ignore_buttons_timer;
//Each suit has 13 cards, occupying bit positions 1-13 of each consecutive 16-bit word. Bits 0,14,15 unused
byte spent_cards[16];   

struct player_state {
   byte cards[10];       //current hand (0-4=down cards, 5-9=up cards)
   byte score;           //score for that hand
   bool has_rf_nine;     //has a 9-card running flush 
   bool has_rf_ten;      //has a 10-card running flush
};

struct character_state {
   word money;           //current winnings (in dollars)
   byte type;            //type of character (wizard, orangutan, dwarf, etc.
   char name[14];
   byte lives;
};

byte menu_pos;   //Used to track position of cursor in states that have a cursor
byte cursor_pos; //Used to track position of cursor in states that have a cursor
word cur_pot;    //current 'pot' being bet
word new_bet;    //how much should we add to the pot?

#define NUM_PLAYERS    2

player_state    players[NUM_PLAYERS];
character_state char_state; 


//Characters are 6x8, so 1 card == two characters == 12x8
//Screen is 128x64  -> it can have 10 cards visible in a single row (MAX), with probably 6 rows max
//Initially, we'll only be able to play against the dealer - a hand is five cards face-up and five cards face-down, 
//with all ten face-down for the dealer
//
//The player will have two rows of five cards each - with each card being 16 x 12 (total cards being 80 by 24, minimum)
//The player + dealer would be 80 x 48 (vs a total screen real estate of 128 x 64, which gives you room for some empty 
//space, borders, etc. and other niceties
#define card_w  16
#define card_h  12


//////////////////////////////////////////////////////////////
//            Helper Functions                              //
//////////////////////////////////////////////////////////////

//Figure out how many trailing spaces in the character's name
byte trailing_spaces() {
  byte spaces = 0;
  for(byte i=0; i<14; i++) {
    if(char_state.name[13-i]==32)
      spaces++;
    else
      break; 
  }
  return spaces;
  
};
//Return limit if val exceeds limit (used to score face cards as 10)
byte iceil(byte limit, byte val)
{
  if(val > limit)
    return limit;
  else
    return val; 
}

//Flip Pixel Values
void invert_region(byte x, byte y, byte w, byte h)
{
  for(byte i=x; i<x+w; i++)
  {
    for(byte j=y; j<y+h; j++)
    {
      byte pix = 1-arduboy.getPixel(i,j);
      arduboy.drawPixel(i,j, pix); 
    }
  }
}

//Shuffle the card deck to start a new hand
void shuffle_deck()
{
  //First, kick the random seed value, just for good measure
  arduboy.initRandomSeed();
  //Each suit has 13 cards, occupying bit positions 1-13 of each consecutive 16-bit word. Bits 0,14,15 unused.
  //The 'spent_cards' array has a 1 in the bit position of cards that are 'taken' and can't be randomly dealt. 
  //To 'shuffle' the deck, we reset things by only setting bits for non-existant cards
  for (byte i=0; i<16; i++) {
    if((i & 0x1)!=0)
      spent_cards[i] = 0xC0;
    else
      spent_cards[i] = 0x01;
  }
}

//Deal a card and mark it 'used' in the deck
byte deal_new_card()
{
    long new_card;
    bool card_taken = true;
    while(card_taken)
    {
      //randomly select a new card
      new_card = random(0,128);
      //Check if it's taken
      byte index = ((byte) new_card) >> 3; 
      card_taken = (spent_cards[index] & (0x01 << (((byte)new_card) & 0x07))) != 0;
    }
    //Set the 'taken' bit in the array
    spent_cards[((byte)new_card) >> 3] |= (0x01 << (((byte)new_card) & 0x07));

    //Format the card - suit in upper 4 bits, card val in lower 4 bits
    byte formatted_card = (byte)new_card;
    //Serial.println(int(formatted_card));
    return formatted_card;
}

//Deal out five cards to each player to start a hand
void initial_deal()
{
    shuffle_deck();
    selected_card = 0;
    tagged_cards  = 0;
    //Deal the player's and dealer's hand
    for(int i=0; i<5; i++)
    {
       players[0].cards[i] = deal_new_card();   //user
       players[1].cards[i] = deal_new_card();   //dealer
    }
}



//Draw a given player's hand to the screen
void draw_hand(byte player, bool show_lower, bool upper_visible, byte sel, word tag)
{
  byte v_offset;
  //Draw the lower 5 cards
  if(player==0)
    v_offset = 51;
  else if(player==1)
    v_offset = 1;
  for(byte i=0; i<5; i++)
  {
    arduboy.drawRoundRect(5+card_w*i, v_offset,   card_w, card_h, 2, 1);
    if(show_lower) {
      arduboy.drawChar(5+card_w*i + 2,  v_offset+3, suit_lut[(players[player].cards[i] & 0x70) >> 4], 1, 0, 1);
      arduboy.drawChar(5+card_w*i + 8,  v_offset+3, card_lut[players[player].cards[i] & 0xf],1, 0, 1);
      //Draw highlighted card
      if(player==0 && sel==i)
        arduboy.drawRoundRect(5+card_w*i-1,v_offset-1, card_w+2, card_h+2, 2, 1);
      if(tag & (1 << i))
        invert_region(5+card_w*i,v_offset,card_w,card_h);
    }
  }
  
  //Draw the upper 5 cards
  if(upper_visible)
  {
    if(player==0)
      v_offset = 38;
    else if(player==1)
      v_offset = 13;
    for(byte i=0; i<5; i++)
    {
      arduboy.drawRoundRect(5+card_w*i, v_offset,   card_w, card_h, 2, 1);
      arduboy.drawChar(5+card_w*i + 2,  v_offset+3, suit_lut[(players[player].cards[5+i] & 0x70) >> 4], 1, 0, 1);
      arduboy.drawChar(5+card_w*i + 8,  v_offset+3, card_lut[players[player].cards[5+i] & 0xf],1, 0, 1);
      //Draw highlighted card
      if(player==0 && sel==(5+i))
        arduboy.drawRoundRect(5+card_w*i-1,v_offset-1, card_w+2, card_h+2, 2, 1);
      if(tag & (1 << (5+i)))
        invert_region(5+card_w*i,v_offset,card_w,card_h);
    }
  }
}


byte evaluate_player_hand(byte player)
{
  //Sort the player's hand  
  byte sorted_cards[10];
  byte lowest_card = 255;
  byte last_lowest = 0;
  //Loop over every entry in our new array
  for(byte i=0; i<10; i++)
  {
    lowest_card = 255;
    //Loop over the whole previous array
    for(byte j=0; j<10; j++)
    {
      if(players[player].cards[j] < lowest_card)
      {
        if(players[player].cards[j] > last_lowest)
           lowest_card = players[player].cards[j];
      }
    }
    sorted_cards[i] = lowest_card;
    last_lowest     = lowest_card;
  }
  //sorted_cards[10] now contains a sorted low-to-high version of the player's hand
  byte suit_count[8]  = {0,0,0,0,0,0,0,0};
  byte card_count[14] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};

  //Now loop over cards to collect some statistics for scoring our hand
  for(byte i=0; i<10; i++)
  {
    byte suit = (sorted_cards[i] & 0x70) >> 4;
    byte card = (sorted_cards[i] & 0x0F);
    suit_count[suit] += 1;
    card_count[card] += 1;
  }
  
  byte face_count = card_count[11] + card_count[12] + card_count[13];
  byte hand_sum = 0;

  Serial.print("Evaluating player:");
  Serial.print(player, DEC);
  Serial.print("...\n");
  Serial.print("Card_count:");
  for(byte i=0; i<14; i++) {
    Serial.print(card_count[i],DEC);
    Serial.print(" ");
  }
  Serial.print("\n");
  Serial.print("Face_count:");
  Serial.println(face_count,DEC);


  //Before we evaluate the hand, check if the player has a 9 or 10-card running flush, and set the flags accordingly
  //(this is used to evaluate whether or not this player can 'cripple mr onion' at a later point
  /*Crippling Mr Onion: if a player displays a Great Onion, an opponent may display a 
    nine-card running flush and instantly win the hand. If a player displays a Great 
    or Lesser Onion, an opponent may display a ten-card running flush and instantly win 
    the hand; a player with a ten-card running flush can also use it to steal the win 
    from someone who has previously crippled Mr Onion with a nine-card running flush. 
    (This is the only non-optional modifier.)
  */
  byte eligible_suit = 8;  //8 is an 'illegal' value, so we'll initialize it to that so we know if it doesn't get set
  for(byte i=0; i<8; i++) {
    if(suit_count[i]>8)
      eligible_suit = i;
  }
  //If we don't have an eligible suit, just set the flags false and evaluate like normal
  if(eligible_suit==8) {
    players[player].has_rf_nine = false;
    players[player].has_rf_ten  = false;
  }
  else {
    //if we have 1 out-of-suit card, remove it temporarily
    byte out_of_suit_card;
    if(suit_count[eligible_suit]==9) {
      for(byte i=0; i<10; i++) {
        byte suit = (sorted_cards[i] & 0x70);
        if(suit =! eligible_suit) {
          out_of_suit_card = sorted_cards[i];    //Let's flag the card that doesn't belong to the suit we're evaluating
        }
      }
      card_count[ (out_of_suit_card & 0x0F) ] -= 1;   //Temporarily subtract the out-of-suit card from our card counts
    }
    //Our card counts now contain only the cards from our suit, so let's find the longest contiguous stretch of cards
    byte cur_run  = 0;
    byte long_run = 0;
    
    for(byte i=0; i<15; i++) {
      byte count;
      if(i==14)
        count = card_count[1];
      else
        count = card_count[i];
      if(count==1)
        cur_run += 1;
      else {
        if(cur_run>long_run)
          long_run = cur_run;
        cur_run = 0;
      }
    }
    //Set flags as needed
    if(long_run==10)
      players[player].has_rf_ten = true;
    else
      players[player].has_rf_ten = false;
    if(long_run==9)
      players[player].has_rf_nine = true;
    else
      players[player].has_rf_nine = false;
      
    //clean up by restoring our card count if needed
    if(suit_count[eligible_suit]==9)
      card_count[ (out_of_suit_card & 0x0F) ] += 1;
  }
  //Let's start with the highest scoring hands first (which are also easy to evaluate)
  // Great Onion, five picture cards and five aces. = 16
  if(card_count[C_ACE]==5 && face_count==5)
    return 16;

  // Lesser Onion, four picture cards and four aces; = 15
  if(card_count[C_ACE]>=4 && face_count>=4)
    return 15;

  // Triple Onion, three picture cards and three aces; = 14
  if(card_count[C_ACE]>=3 && face_count>=3)
    return 14;

  //Double Onion, two picture cards and two aces; = 13
  if(card_count[C_ACE]>=2 && face_count>=2)
    return 13;
      
  //Seven-card Onion - seven cards totalling 21; = 12  
  //This can only be true if any Aces are worth 1, so we can simplify and not worry about if Ace=11.
  //Loop over every 7-card combo and check if it sums to 21
  for(byte i=0;i<(10-2);i++) {
    for(byte j=i+1;j<(10-1);j++) {
      for(byte k=j+1;k<10;k++) {
        hand_sum = 0;
        for(byte l=0;l<10;l++) {
           if( (l!=i) && (l!=j) && (l!=k) )
              hand_sum = hand_sum + iceil(10,(sorted_cards[l] & 0x0F));   
        }
        if(hand_sum==21)
          return 12;
      }
    }
  }

  // Wild Royal, the combination 888 in a hand when eights are wild (see below); = 11
  // NOTE - I'm ignoring the '8s are wild' thing for the moment, FIXME: possibly  revisit this later
  if(card_count[C_EIGHT]>=3)
    return 11;

  //Six-card Onion; = 10 - check out every unique 6 card group for onion-ness
  //FIXME: At the moment, this only considers Ace==1, not Ace==11. Figure out how to check those cases as well
  for(byte i=0;i<(10-3);i++) {
    for(byte j=i+1;j<(10-2);j++) {
      for(byte k=j+1;k<(10-1);k++) {
        for(byte l=k+1;l<10;l++) {
          //Evaluate if 6-card selection is 21  
          hand_sum = 0;
          for(byte m=0;m<10;m++) {
            if( (m!=i) && (m!=j) && (m!=k) && (m!=l) )
              hand_sum = hand_sum + iceil(10,(sorted_cards[m] & 0x0F));   
          }
          if(hand_sum==21)
            return 10;
        }
      }
    }
  }   

  //Royal, the combination 777 of any suit; = 9
  //Just check if our tally of 7s exceeds 2
  if(card_count[7]>2)
    return 9;
  
  //Five-card Onion; = 8  - check out every unique 5 card group for onion-ness.
  //This might take a little while...
  //Fixme; At the moment, this assumes Ace==1, not 11
  for(byte i=0;i<(10-4);i++) {
    for(byte j=i+1;j<(10-3);j++) {
      for(byte k=j+1;k<(10-2);k++) {
        for(byte l=k+1;l<(10-1);l++) {
          for(byte m=l+1;m<10;m++) {
            //Evaluate if 5-card selection is 21  
            hand_sum = 0;
            for(byte n=0;n<10;n++) {
              if( (n==i) || (n==j) || (n==k) || (n==l) || (n==m) )  
                hand_sum = hand_sum + iceil(10,(sorted_cards[n] & 0x0F));   
            }
            if(hand_sum==21)
              return 8;
          }
        }
      }
    }
  }   

  //Broken Royal, the combination 678 of any suit; = 7
  if( (card_count[6] > 0) && (card_count[7] > 0) && (card_count[8]>0) )
    return 7;

  //Four-card Onion, as two- and three-card Onion above; = 6  - check out every unique 4 card group for onion-ness
  //FIXME: At the moment, this only considers Ace==1, not Ace==11. Figure out how to check those cases as well
  for(byte i=0;i<(10-3);i++) {
    for(byte j=i+1;j<(10-2);j++) {
      for(byte k=j+1;k<(10-1);k++) {
        for(byte l=k+1;l<10;l++) {
          //Evaluate if 4-card selection is 21  
          hand_sum = 0;
          for(byte m=0;m<10;m++) {
            if( (m==i) || (m==j) || (m==k) || (m==l) )
              hand_sum = hand_sum + iceil(10,(sorted_cards[m] & 0x0F));   
          }
          if(hand_sum==21)
            return 6;
        }
      }
    }
  }   

  //Flush, a set of three or more cards totalling between 16 and 21 inclusive, with all in the same suit-pair = 5 
  //FIXME: consider Ace==11 case. Also, consider how to rank two different flushes somehow    
  //loop over all suits
  for(byte suit=0;suit<(NUM_SUITS/2);suit++) {
    //For each suit, we need to see what possible combinations there are. For now, we'll do a naive
    //implementation - because the cards are sorted, start summing the cards in each suit, and if you reach
    //above 16 (but below 21) with 3 or more cards, just report it as a flush. We won't try to rank specific
    //flushes at this point
    hand_sum = 0;
    byte card_count = 0;
    for(byte i=0;i<10;i++) {
      if( ((sorted_cards[i] & 0xF0) >> 5)==suit) {
        hand_sum   = hand_sum + iceil(10,(sorted_cards[i] & 0x0F));
        card_count = card_count + 1;
        if( (card_count > 2) && (hand_sum > 15) && (hand_sum < 22) )
          return 5; 
      }
    }
  }

  //Three-card Onion, three cards with values totalling 21; = 4      
  for(byte i=0;i<(10-2);i++) {
    for(byte j=i+1;j<(10-1);j++) {
      for(byte k=j+1;k<10;k++) {
        hand_sum = 0;
        for(byte l=0;l<10;l++) {
           if( (l==i) || (l==j) || (l==k) )
              hand_sum = hand_sum + iceil(10,(sorted_cards[l] & 0x0F));   
        }
        if(hand_sum==21)
          return 4;
      }
    }
  }

  //Broken Flush, a set of three or more cards totalling between 16 and 21 inclusive, 
  //  and with all but one in the same suit-pair; = 3
  //We'll calculate this just like we did with a regular flush, but with an extra loop trying 
  //each card that *doesn't* match the suit, to see if that pushes it over. At this point we already
  //know that it's not a real flush (or we would have exited already), so we don't need to worry about that  

  for(byte suit=0;suit<(NUM_SUITS/2);suit++) {
    //For each suit, we need to see what possible combinations there are. For now, we'll do a naive
    //implementation - because the cards are sorted, start summing the cards in each suit, and if you reach
    //above 16 (but below 21) with 3 or more cards, just report it as a flush. We won't try to rank specific
    //flushes at this point
    hand_sum = 0;
    byte card_count = 0;
    for(byte i=0;i<10;i++) {
      if( ((sorted_cards[i] & 0xF0) >> 5)==suit) {
        hand_sum   = hand_sum + iceil(10,(sorted_cards[i] & 0x0F));
        card_count = card_count + 1;
        //Now loop over all non-matching-suit cards and test them one at a time
        for(byte j=0;j<10;j++) {
          if( ((sorted_cards[j] & 0xF0) >> 5)!=suit) {
            byte broken_sum = hand_sum + iceil(10,(sorted_cards[j] & 0x0F));
            if( (card_count > 1) && (broken_sum > 15) && (broken_sum <22) )
              return 3;
          }
        }
      }
    }
  }


 //Two-card Onion, two cards with values totalling 21; = 2
 //FIXME: This could be sped up by just looking for Ace+(face or 10) from card counts
 for(byte i=0;i<(10-1);i++) {
  for(byte j=i;j<(10);j++) {
    hand_sum = 0;
    for(byte k=0;k<10;k++) {
      if( (k==i) || (k==j) ) {
        byte tmp_card_val = iceil(10,(sorted_cards[k] & 0x0F));
        if(tmp_card_val == 1)
          hand_sum = hand_sum + 11;
        else
          hand_sum = hand_sum + tmp_card_val;

        if(hand_sum==21)
          return 2;
      }
    }
  }
 }

  //Bagel - two cards with values totalling 20; = 1
  //FIXME: This could be sped up by just looking for Ace+9 or 2x(Face card or 10) from card counts
 for(byte i=0;i<(10-1);i++) {
  for(byte j=i;j<(10);j++) {
    hand_sum = 0;
    for(byte k=0;k<10;k++) {
      if( (k==i) || (k==j) ) {
        byte tmp_card_val = iceil(10,(sorted_cards[k] & 0x0F));
        if(tmp_card_val == 1)
          hand_sum = hand_sum + 11;
        else
          hand_sum = hand_sum + tmp_card_val;

        if(hand_sum==20)
          return 2;
      }
    }
  }
 }
  //Nada!
  return 0;
}


void print_player_hand(byte score)
{
  switch(score)
  {       
    case 0:arduboy.print("worthless hand!");
           break;
    case 1:arduboy.print("Bagel");
           break;
    case 2:arduboy.print("2-Card Onion");
           break;
    case 3:arduboy.print("Broken Flush");
           break;
    case 4:arduboy.print("3-Card Onion");
           break;
    case 5:arduboy.print("Flush");
           break;
    case 6:arduboy.print("4-Card Onion");
           break;
    case 7:arduboy.print("Broken Royal");
           break;
    case 8:arduboy.print("5-Card Onion");
           break;
    case 9:arduboy.print("Royal");
           break;
    case 10:arduboy.print("6-Card Onion");
           break;
    case 11:arduboy.print("Wild Royal");
            break;
    case 12:arduboy.print("7-Card Onion");
            break;
    case 13:arduboy.print("Double Onion");
            break;
    case 14:arduboy.print("Triple Onion");
            break;
    case 15:arduboy.print("Lesser Onion");
            break;
    case 16:arduboy.print("Great Onion");
            break;
  }
  
}

byte bitsum(word i)
{
  byte sum = 0;
  for(byte j=0; j<16; j++)
     if(i & (1 << j))
        sum++;
  return sum;
}

byte ffs(word &i, bool clear)
{
  for(byte j=0; j<16; j++)
    if(i & (1 << j))
    {
      i = i ^ (1 << j);
      return j;
    }
  return 255; 
}

void draw_character(byte x, byte y) {

     switch(char_state.type)
   {
      case(0):arduboy.drawSlowXYBitmap(x, y, dwarf_bmp, 32, 32, WHITE);
              break;
      case(1):arduboy.drawSlowXYBitmap(x, y, gnome_bmp, 32, 32, WHITE);
              break;
      case(2):arduboy.drawSlowXYBitmap(x, y, werewolf_bmp, 32, 32, WHITE);
              break;
      case(3):arduboy.drawSlowXYBitmap(x, y, troll_bmp, 32, 32, WHITE);
              break;
      case(4):arduboy.drawSlowXYBitmap(x, y, street_urchin_bmp, 32, 32, WHITE);
              break;
      case(5):arduboy.drawSlowXYBitmap(x, y, golem_bmp, 32, 32, WHITE);
              break;
      case(6):arduboy.drawSlowXYBitmap(x, y, goblin_bmp, 32, 32, WHITE);
              break;
      case(7):arduboy.drawSlowXYBitmap(x, y, witch_bmp, 32, 32, WHITE);
              break;
      case(8):arduboy.drawSlowXYBitmap(x, y, wizard_bmp, 32, 32, WHITE);
              break;
      case(9):arduboy.drawSlowXYBitmap(x, y, librarian_bmp, 32, 32, WHITE);
              break;
      case(10):arduboy.drawSlowXYBitmap(x, y, thief_bmp, 32, 32, WHITE);
              break;
      case(11):arduboy.drawSlowXYBitmap(x, y, assassin_bmp, 32, 32, WHITE);
              break;
   }
   //Display character sprite
   arduboy.drawRoundRect(x,y, 32, 32, 2);
  
}

/////////////////////////////////////////////////////////////////////////////
//                                                                         // 
//                   State Evaluation Functions                            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//The overall 'game' just walks through a number of different states. The 
//logic for each state is encapsulated in a state evaluation function called
//"do_state_X()" that handles all display and button functions


////////////////////////////////////////////////////
//         Display the Start Screen               //
////////////////////////////////////////////////////
void do_state_start_screen() {
   //Let's draw the start-screen logo until the user presses the "A" button
   arduboy.drawSlowXYBitmap(0, 0, title_screen, 128, 64, WHITE);
   if(!ignore_buttons) {
    if(arduboy.pressed(A_BUTTON)) {
      ignore_buttons = true;
      ignore_buttons_timer = 15;
      state    = STATE_FHI;
      substate = 0;
      menu_pos = 0;
    }
   }
   else if (ignore_buttons_timer > 0)
    ignore_buttons_timer -= 1;
   else
    ignore_buttons = false;
  
}

////////////////////////////////////////////////////
//         Display FHI Tag                        //
////////////////////////////////////////////////////
void do_state_fhi() {
   arduboy.setCursor(32,16);
   arduboy.print(F("A Product of"));
   arduboy.setCursor(32,24);
   arduboy.print(F("Fenton Heavy"));
   arduboy.setCursor(36,32);
   arduboy.print(F("Industries"));
  
   if(!ignore_buttons) {
    if(arduboy.pressed(A_BUTTON)) {
      ignore_buttons = true;
      ignore_buttons_timer = 15;
      state    = STATE_MENU;
      substate = 0;
      menu_pos = 0;
    }
   }
   else if (ignore_buttons_timer > 0)
    ignore_buttons_timer -= 1;
   else
    ignore_buttons = false;
  
}

///////////////////////////////////////////////////
//      Display the Action Menu                  //
///////////////////////////////////////////////////
void do_state_menu() {
   //Draw the Menu
   arduboy.setCursor(7,6);
   arduboy.print(F("Ankh-Morpork Casino"));
   arduboy.drawFastHLine(16,15,96);
   arduboy.setCursor(16,20);
   if(char_state.money>=25)
      arduboy.print(F("Play a hand"));
   else
      arduboy.print(F("Sell Sausages"));
   arduboy.setCursor(16,28);
   arduboy.print(F("Edit Character"));
   arduboy.setCursor(16,36);
   arduboy.print(F("Read the rules"));
   arduboy.setCursor(16,44);
   arduboy.print(F("Save to EEPROM"));
   arduboy.setCursor(16,52);
   arduboy.print(F("Reset Game"));
   
   //Draw the 'cursor'
   byte vpos = 0;
   switch(menu_pos) {
      case(0):vpos = 20;
              break;
      case(1):vpos = 28;
              break;
      case(2):vpos = 36;
              break;
      case(3):vpos = 44;
              break;
      case(4):vpos = 52;
              break;
      default:vpos = 20;
   }
   arduboy.setCursor(8,vpos);
   arduboy.print(">");
   
   if(!ignore_buttons) {
      if(arduboy.pressed(UP_BUTTON))
      {
        if(menu_pos > 0) {
          menu_pos = menu_pos - 1;
          ignore_buttons = true;
          ignore_buttons_timer = 10;
        }
      }  
      else if(arduboy.pressed(DOWN_BUTTON))
      {
          if(menu_pos < 4) { 
            menu_pos = menu_pos + 1;
            ignore_buttons = true;
            ignore_buttons_timer = 10;
          }
      }
      else if(arduboy.pressed(A_BUTTON))
      {
          //Play a hand
          if(menu_pos==0)
          {
              substate = 0;
              ignore_buttons = true;
              ignore_buttons_timer = 25;
              if(char_state.money >= 25) {
                initial_deal();
                cur_pot = 25;             //initial bet
                char_state.money = char_state.money-25;   //subtract initial bet from money
                new_bet = 0;              //this will track any additional amount we bet after the first draw
                state = STATE_1ST_DRAW;
              }
              else {
                state = STATE_LOSE;
              }
          }
          //Edit Character
          else if(menu_pos==1)
          {
            substate   = 0;
            menu_pos   = 0;
            cursor_pos = 0;
            ignore_buttons = true;
            ignore_buttons_timer = 20;
            state = STATE_CHARACTER;
          }
          //Read the rules
          else if(menu_pos==2)
          {
            substate   = 0;
            menu_pos   = 0;
            cursor_pos = 0;
            ignore_buttons = true;
            ignore_buttons_timer = 20;
            state = STATE_RULES;
          }
          //Save current player stats to EEPROM
          else if(menu_pos==3)
          {
            invert_region(0,0,128,64);    //Blink screen just to show activity
            EEPROM.write(EEPROM_STORAGE_SPACE_START + 8, (char_state.money & 0xFF));       //money[0]
            EEPROM.write(EEPROM_STORAGE_SPACE_START + 9, (char_state.money & 0xFF00) >> 8); //money[1]
            EEPROM.write(EEPROM_STORAGE_SPACE_START + 10, char_state.type);                //type
            for(int i=0; i<14; i++) {
              EEPROM.write(EEPROM_STORAGE_SPACE_START + 11 + i, char_state.name[i]);        //name
            }
            ignore_buttons = true;
            ignore_buttons_timer = 10;
          }
          //Reset character
          else if(menu_pos==4)
          { 
            invert_region(0,0,128,64);    //Blink screen just to show activity
            char_state.money = 500;
            char_state.type  = random(0,12);   //Choose a random player type
            char_state.name[0] = 'A';
            char_state.name[1] = 'N';
            char_state.name[2] = 'K';
            char_state.name[3] = 'H';
            char_state.name[4] = 'M';
            char_state.name[5] = 'O';
            char_state.name[6] = 'R';
            char_state.name[7] = 'P';
            char_state.name[8] = 'O';
            char_state.name[9] = 'R';
            char_state.name[10]= 'K';
            char_state.name[11]= 'I';
            char_state.name[12]= 'A';
            char_state.name[13]= 'N';
          }
      } 
    }
    else if (ignore_buttons_timer > 0)
      ignore_buttons_timer -= 1;
    else
      ignore_buttons = false;

}

///////////////////////////////////////////////////////////////////////////
//       View and Configure Character                                    //
///////////////////////////////////////////////////////////////////////////
void do_state_character()
{
   arduboy.setCursor(9,8);
   arduboy.print("Name:");
   for(int i=0; i<14; i++)
      arduboy.print(char_state.name[i]);
      
   arduboy.setCursor(9,16);
  //Display character information
   arduboy.print("Type:");
   draw_character(92, 28);
   switch(char_state.type)
   {
      case(0):arduboy.print("Dwarf");          //pic
              break;
      case(1):arduboy.print("Gnome");          //pic
              break;
      case(2):arduboy.print("Werewolf");       //Pic - but can probably be improved a lot
              break;
      case(3):arduboy.print("Troll");          //pic
              break;
      case(4):arduboy.print("Street Urchin");  //pic
              break;
      case(5):arduboy.print("Golem");          //pic
              break;
      case(6):arduboy.print("Goblin");         //pic
              break;
      case(7):arduboy.print("Witch");          //pic
              break;
      case(8):arduboy.print("Wizard");         //pic
              break;
      case(9):arduboy.print("Librarian");      //pic
              break;
      case(10):arduboy.print("Thief");         //pic
              break;
      case(11):arduboy.print("Assassin");      //pic
              break;
   }

   
   if(menu_pos==0)
   {
      invert_region(cursor_pos*6 + 38,8,6,8);
   }
   else if(menu_pos==1)
   {
      invert_region(cursor_pos*6 + 38,16,6,8);
   }
   arduboy.setCursor(9,24);
   arduboy.print("Back");
   arduboy.drawFastHLine(7,35,32);

  //Draw the 'cursor'
   byte vpos = 0;
   switch(menu_pos) {
      case(0):vpos = 8;
              break;
      case(1):vpos = 16;
              break;
      case(2):vpos = 24;
              break;
      default:vpos = 16;
   }
   arduboy.setCursor(2,vpos);
   arduboy.print(">");

   if(!ignore_buttons) {
      if(arduboy.pressed(UP_BUTTON))
      {
        if(menu_pos > 0) {
          menu_pos = menu_pos - 1;
          ignore_buttons = true;
          ignore_buttons_timer = 10;
        }
      }  
      else if(arduboy.pressed(DOWN_BUTTON))
      {
          if(menu_pos < 2) {
            cursor_pos = 0; 
            menu_pos = menu_pos + 1;
            ignore_buttons = true;
            ignore_buttons_timer = 10;
          }
      }
      else if(arduboy.pressed(RIGHT_BUTTON))
      {
          if(menu_pos==0 and cursor_pos<13) {
            cursor_pos += 1;
            ignore_buttons = true;
            ignore_buttons_timer = 5;
          }
      }
      else if(arduboy.pressed(LEFT_BUTTON))
      {
          if(menu_pos==0 and cursor_pos>0) {
            cursor_pos -= 1;
            ignore_buttons = true;
            ignore_buttons_timer = 5;
          }
      }
      else if(arduboy.pressed(A_BUTTON))
      {
          //Change Character Name
          if(menu_pos==0)
          {
            ignore_buttons = true;
            ignore_buttons_timer = 10;
            if(char_state.name[cursor_pos]==65)
               char_state.name[cursor_pos]=32;
            else if(char_state.name[cursor_pos]==32)
               char_state.name[cursor_pos]=90;
            else
               char_state.name[cursor_pos] -= 1;
          }
          //Change type
          else if(menu_pos==1)
          {
            ignore_buttons = true;
            ignore_buttons_timer = 10;
            if(char_state.type > 0)
               char_state.type -= 1;
            else if(char_state.type==0)
               char_state.type = 11;
          }
          //Return to menu
          else if(menu_pos==2)
          {
              ignore_buttons = true;
              ignore_buttons_timer = 20;
              initial_deal();
              state = STATE_MENU;
          }
      }
      else if(arduboy.pressed(B_BUTTON))
      {
          if(menu_pos==0)
          {
            ignore_buttons = true;
            ignore_buttons_timer = 10;
            if(char_state.name[cursor_pos]==90)
               char_state.name[cursor_pos]=32;
            else if(char_state.name[cursor_pos]==32)
               char_state.name[cursor_pos]=65;
            else
               char_state.name[cursor_pos] += 1;
          }
          else if(menu_pos==1)
          {
            ignore_buttons = true;
            ignore_buttons_timer = 10;
            if(char_state.type < 11)
               char_state.type += 1;
            else if(char_state.type==11)
               char_state.type = 0;
          }
      }
   }
   else if (ignore_buttons_timer > 0)
     ignore_buttons_timer -= 1;
   else
     ignore_buttons = false;

   arduboy.setCursor(3,42);
   arduboy.print(F("Money:"));
   arduboy.print(char_state.money);
   arduboy.setCursor(3,50);
   arduboy.print(F("Rank:"));
   if (char_state.money < 1000)
     arduboy.print(F("Begger"));
   else if(char_state.money < 2000)
     arduboy.print(F("Shoveler"));
   else if(char_state.money < 5000)
     arduboy.print(F("Clacker"));
   else if(char_state.money < 7500)
     arduboy.print(F("City Watch"));
   else if(char_state.money < 15000)
     arduboy.print(F("Troll King"));
   else if(char_state.money < 25000)
     arduboy.print(F("Orangutan"));
   else if(char_state.money < 35000)
     arduboy.print(F("Commander"));
   else if(char_state.money > 60000)
     arduboy.print(F("Patrician"));
}

///////////////////////////////////////////////////////////////////////////
//             Display the rules                                         //
///////////////////////////////////////////////////////////////////////////
void do_state_rules()
{
  switch(substate) {
    case(0):arduboy.setCursor(16,8);
            arduboy.print(F("Cripple Mr Onion"));
            arduboy.setCursor(5,20);
            arduboy.print(F("Win money & climb up"));
            arduboy.setCursor(5,28);
            arduboy.print(F("Ankh-Morporks social"));
            arduboy.setCursor(5,36);
            arduboy.print(F("ladder, or lose it"));
            arduboy.setCursor(5,44);
            arduboy.print(F("and be on the street"));
            arduboy.setCursor(5,52);
            arduboy.print(F("selling sausages..."));
            break;
    case(1):arduboy.setCursor(50,8);
            arduboy.print(F("Rules"));
            arduboy.setCursor(6,20);
            arduboy.print(F("Players get 5 cards,"));
            arduboy.setCursor(6,28);
            arduboy.print(F("face-down, and can"));
            arduboy.setCursor(6,36);
            arduboy.print(F("trade in up to 4."));
            arduboy.setCursor(6,44);
            arduboy.print(F("5 more face-up"));
            arduboy.setCursor(6,52);
            arduboy.print(F("complete the hand."));
            break;
            /*
            #define STAVES    0      //STAVES   = \ = 92
            #define SWORDS    1      //SWORDS   = T = 84
            #define CUPS      2      //CUPS     = Y = 89
            #define COINS     3      //COINS    = o = 111
            #define OCTOGRAMS 4      //OCTOGRAM = * = 42
            #define ELEPHANTS 5      //ELEPHANT = J = 74
            #define TERRAPINS 6      //TERRAPINS= & = 38  @=64
            #define CROWNS    7      //CROWNS   = # = 35
            //The suits are grouped into pairs:  Staves and swords (0), cups and coins (1), octograms and elephants (2), terrapins and crowns (3)
            */
    case(2):arduboy.setCursor(12,8);
            arduboy.print(F("CMO uses 8 suits,"));
            arduboy.setCursor(10,16);
            arduboy.print(F("paired as follows:"));
            arduboy.setCursor(6,28);
            arduboy.print(F("Staves (\\) and"));
            arduboy.setCursor(6,36);
            arduboy.print(F("   Swords (T)"));
            arduboy.setCursor(6,46);
            arduboy.print(F("Cups (Y) and"));
            arduboy.setCursor(6,54);
            arduboy.print(F("   Coins (o)"));
            break;
    case(3):arduboy.setCursor(6,8);
            arduboy.print(F("Octograms (*) and"));
            arduboy.setCursor(6,16);
            arduboy.print(F("  Elephants (J)"));
            arduboy.setCursor(6,30);
            arduboy.print(F("Terrapins (&) and"));
            arduboy.setCursor(6,38);
            arduboy.print(F("   Crowns (#)"));
            break;
    case(4):arduboy.setCursor(6,20);
            arduboy.print(F("Valid hands are"));
            arduboy.setCursor(6,28);
            arduboy.print(F("listed in ascending"));
            arduboy.setCursor(6,36);
            arduboy.print(F("order..."));
            break;
    case(5):arduboy.setCursor(50,8);
            arduboy.print(F("Hands"));
            arduboy.setCursor(6,20);
            arduboy.print(F("Bagel:2 cards with"));
            arduboy.setCursor(6,28);
            arduboy.print(F("values totalling 20"));
            arduboy.setCursor(6,36);
            arduboy.print(F("2-card Onion:2 cards"));
            arduboy.setCursor(6,44);
            arduboy.print(F("totalling 21"));
            arduboy.setCursor(6,52);
            arduboy.print("...");
            break;
    case(6):arduboy.setCursor(6,8);
            arduboy.print(F("Broken Flush: a set"));
            arduboy.setCursor(6,16);
            arduboy.print(F("of 3 or more cards"));
            arduboy.setCursor(6,24);
            arduboy.print(F("totalling 16-21"));
            arduboy.setCursor(6,32);
            arduboy.print(F("inclusive, with all")); 
            arduboy.setCursor(6,40);
            arduboy.print(F("but one in the same"));
            arduboy.setCursor(6,48);
            arduboy.print(F("suit-pair"));
            arduboy.setCursor(6,56);
            arduboy.print("...");
            break;
    case(7):arduboy.setCursor(6,8);
            arduboy.print(F("3-Card Onion:3 cards"));
            arduboy.setCursor(6,16);
            arduboy.print(F("totalling 21"));
            arduboy.setCursor(6,24);
            arduboy.print(F("Flush:A broken flush"));
            arduboy.setCursor(6,32);
            arduboy.print(F("but with all cards"));
            arduboy.setCursor(6,40);
            arduboy.print(F("in same suit-pair"));
            arduboy.setCursor(6,48);
            arduboy.print("...");
            break;
    case(8):arduboy.setCursor(6,8);
            arduboy.print(F("4-card Onion:..."));
            arduboy.setCursor(6,16);
            arduboy.print(F("Broken Royal:a combo"));
            arduboy.setCursor(6,24);
            arduboy.print(F("of 678, in any suit"));
            arduboy.setCursor(6,32);
            arduboy.print(F("5-card Onion:..."));
            arduboy.setCursor(6,40);
            arduboy.print(F("Royal:combo of 777"));
            arduboy.setCursor(6,48);
            arduboy.print(F("in any suit"));
            arduboy.setCursor(6,56);
            arduboy.print("...");
            break;
    case(9):arduboy.setCursor(6,8);
            arduboy.print(F("6-card Onion:..."));
            arduboy.setCursor(6,16);
            arduboy.print(F("Wild Royal:combo of"));
            arduboy.setCursor(6,24);
            arduboy.print(F("888 in a hand"));
            arduboy.setCursor(6,32);
            arduboy.print(F("7-card Onion:..."));
            arduboy.setCursor(6,40);
            arduboy.print("...");
            break;
    case(10):arduboy.setCursor(6,8);
            arduboy.print(F("Double Onion:2 face")); 
            arduboy.setCursor(6,16);
            arduboy.print(F("cards & 2 Aces"));
            arduboy.setCursor(6,24);             
            arduboy.print(F("Triple Onion:3 face"));
            arduboy.setCursor(6,32);
            arduboy.print(F("cards & 3 Aces"));
            arduboy.setCursor(6,40);
            arduboy.print("...");
            break;
    case(11):arduboy.setCursor(6,8);
            arduboy.print(F("Lesser Onion:4 face"));
            arduboy.setCursor(6,16);
            arduboy.print(F("cards & 4 Aces"));
            arduboy.setCursor(6,24);
            arduboy.print(F("Great Onion:5 face"));
            arduboy.setCursor(6,32);
            arduboy.print(F("cards & 5 Aces"));
            break; 
  }
  //Handle buttons
   if(!ignore_buttons) {
     if(arduboy.pressed(A_BUTTON)) {
       ignore_buttons = true;
       ignore_buttons_timer = 20;
       substate++;
       if(substate > 11)
          state = STATE_MENU;
     }
   }
  else if (ignore_buttons_timer > 0)
    ignore_buttons_timer -= 1;
  else
    ignore_buttons = false;
}

///////////////////////////////////////////////////////////////////////////
//       Draw the initial hand dealt to each player                      //
///////////////////////////////////////////////////////////////////////////
void do_state_1st_draw()
{
  //Draw player's hand
  if(!ignore_buttons)
  {
    if (arduboy.pressed(A_BUTTON)) {
       ignore_buttons = true;
       ignore_buttons_timer = 20;
       state = STATE_BET;
       //Deal replacement cards
       for(byte i=0; i<5; i++)
          if(tagged_cards & (1 << i))
            players[0].cards[i] = deal_new_card();
       //Deal upper 5 cards
       for(byte i=0; i<5; i++)
       {
          players[0].cards[5+i]=deal_new_card();  //user
          players[1].cards[5+i]=deal_new_card();  //dealer
       }
       selected_card = 0;
       tagged_cards  = 0;
    }
    else if (arduboy.pressed(B_BUTTON)) {
      if(bitsum(tagged_cards)<4)
        tagged_cards = tagged_cards^(1 << selected_card);
      else if(bitsum(tagged_cards)==4 && (tagged_cards & (1 << selected_card)))
        tagged_cards = tagged_cards^(1 << selected_card);
      ignore_buttons = true;
      ignore_buttons_timer = 10;
    }
    else if (arduboy.pressed(LEFT_BUTTON) && selected_card > 0) {
      selected_card -= 1;
      ignore_buttons = true;
      ignore_buttons_timer = 10;
    }
    else if(arduboy.pressed(RIGHT_BUTTON) && selected_card < 4) {
      selected_card += 1;
      ignore_buttons = true;
      ignore_buttons_timer = 10;
    }
  }
  else if (ignore_buttons_timer > 0)
    ignore_buttons_timer -= 1;
  else
    ignore_buttons = false;
  draw_hand(0, true, false, selected_card, tagged_cards);
  //Draw dealer's hand
  draw_hand(1, false, false, 255, 0);

  //Draw current gold level
  arduboy.setCursor(90,54);
  arduboy.print("$");
  arduboy.print(char_state.money);

  //Draw the current 'pot' (what the player has bet)
  arduboy.setCursor(8, 28);
  arduboy.print("Bet:");
  arduboy.print(cur_pot);

  draw_character(90,20);
}

///////////////////////////////////////////////////////////////////////////
//           Bet based on your replacement card                          //
///////////////////////////////////////////////////////////////////////////
void do_state_bet()
{
  if(!ignore_buttons)
  {
    if (arduboy.pressed(A_BUTTON)) {
       ignore_buttons = true;
       ignore_buttons_timer = 20;
       cur_pot = cur_pot + new_bet;
       new_bet = 0;
       //Calculate the score for each player's hand
       state = STATE_2ND_DRAW;
    }
    else if(arduboy.pressed(UP_BUTTON)) {
      ignore_buttons = true;
      ignore_buttons_timer = 5; 
      if( (char_state.money >= 25) && (new_bet < 500) )
      { 
        new_bet += 25;
        char_state.money -= 25;
      }
    }
    else if(arduboy.pressed(DOWN_BUTTON)) {
      ignore_buttons = true;
      ignore_buttons_timer = 5;
      if (new_bet >= 25) {
        new_bet -= 25;
        char_state.money += 25;
      }
    }
  }
  else if (ignore_buttons_timer > 0)
    ignore_buttons_timer -= 1;
  else
    ignore_buttons = false;

  draw_hand(0, true, false, 255, 0);
  //Draw dealer's hand
  draw_hand(1, false, false, 255, 0);
  draw_character(90,20);

  //Draw the current 'pot' (what the player has bet)
  arduboy.setCursor(8, 28);
  arduboy.print("Bet:");
  arduboy.print(cur_pot);

  //Draw text box
  arduboy.fillRoundRect(16, 16, 96, 32, 4, BLACK);
  arduboy.drawRoundRect(16, 16, 96, 32, 4);
  arduboy.setCursor(56,20);
  arduboy.print("Raise?");
  arduboy.setCursor(60,32);
  arduboy.print(new_bet);
  //Draw current gold level
  arduboy.setCursor(90,54);
  arduboy.print("$");
  arduboy.print(char_state.money);

}
///////////////////////////////////////////////////////////////////////////
//       Deal the rest of the hand each player                           //
///////////////////////////////////////////////////////////////////////////
void do_state_2nd_draw()
{
    if(!ignore_buttons)
  {
    if (arduboy.pressed(A_BUTTON)) {
       ignore_buttons = true;
       ignore_buttons_timer = 20;
       //Calculate the score for each player's hand
       players[0].score = evaluate_player_hand(0);
       players[1].score = evaluate_player_hand(1);
       state    = STATE_EVAL;
       substate = 0;
    }
    /*
    else if (arduboy.pressed(B_BUTTON)) {
      tagged_cards = tagged_cards^(1 << selected_card);
      ignore_buttons = true;
      ignore_buttons_timer = 10;
    }
    else if (arduboy.pressed(LEFT_BUTTON) && selected_card > 0) {
      selected_card -= 1;
      ignore_buttons = true;
      ignore_buttons_timer = 10;
    }
    else if(arduboy.pressed(RIGHT_BUTTON) && selected_card < 9) {
      selected_card += 1;
      ignore_buttons = true;
      ignore_buttons_timer = 10;
    }
    */
  }
  else if (ignore_buttons_timer > 0)
    ignore_buttons_timer -= 1;
  else
    ignore_buttons = false;
  draw_hand(0, true, true, 255, 0); //selected_card, tagged_cards);
  draw_hand(1, true, true, 255, 0);
  draw_character(90,20);
  //Draw current gold level
  arduboy.setCursor(90,54);
  arduboy.print("$");
  arduboy.print(char_state.money);

  //Draw the current 'pot' (what the player has bet)
  arduboy.setCursor(8, 28);
  arduboy.print("Bet:");
  arduboy.print(cur_pot);

}

///////////////////////////////////////////////////////////////////////////
//       Evaluate who won this round                                     //
///////////////////////////////////////////////////////////////////////////
void do_state_eval() {
  /*Crippling Mr Onion: if a player displays a Great Onion, an opponent may display a 
    nine-card running flush and instantly win the hand. If a player displays a Great 
    or Lesser Onion, an opponent may display a ten-card running flush and instantly win 
    the hand; a player with a ten-card running flush can also use it to steal the win 
    from someone who has previously crippled Mr Onion with a nine-card running flush. 
    (This is the only non-optional modifier.)
    //Great Onion  = 16
    //Lesser Onion = 15
  */
  bool cripple = false;
  byte winner = 0;
  if(players[0].score > players[1].score) {
    //At this point, we suspect our player has won, but we need to make sure his mr onion hasn't been crippled
    if( (players[0].score==16) && players[1].has_rf_nine ) {
       winner  = 1;
       cripple = true;
    }
    else if( (players[0].score==15) && (players[1].has_rf_nine || players[1].has_rf_ten)) {
       winner  = 1;
       cripple = true;
    }
    else
       winner = 0;
  }
  else if(players[0].score == players[1].score) {
    winner = 2;
  }
  else {
    //At this point, we suspect the dealer has won, but we need to make sure his mr onion hasn't been crippled
    if( (players[1].score==16) && players[0].has_rf_nine ) {
       winner  = 0;
       cripple = true;
    }
    else if( (players[1].score==15) && (players[0].has_rf_nine || players[0].has_rf_ten)) {
       winner  = 0;
       cripple = true;
    }
    else
       winner = 1;
  }
    
  //Draw player's hand and evaluate it
  if(substate<150) {
    if(substate<50) {
      arduboy.setCursor(32,24);
      arduboy.print("Wait while Hex");
      arduboy.setCursor(40,32);
      arduboy.print("evaluates");
      arduboy.setCursor(32,40);
      arduboy.print("your hands...");
    }
    else {
      if( (substate % 10) > 5)
        arduboy.drawSlowXYBitmap(8, 4, hex0_bmp, 112, 56, WHITE);
      else
        arduboy.drawSlowXYBitmap(8, 4, hex1_bmp, 112, 56, WHITE);
    }
    substate += 1;
  }
  else if(substate==150) {
    draw_hand(0, true, true, 0, 0);
    arduboy.setCursor(4,16);
    for(byte i=0;i<(14-trailing_spaces());i++) {
      arduboy.print(char_state.name[i]);  
    }
    arduboy.print(" has a");
    arduboy.setCursor(24,24);
    print_player_hand(players[0].score);
  }
  else if(substate==151) {
    draw_hand(1, true, true, 0, 0);
    arduboy.setCursor(16,32);
    arduboy.print(F("Dealer has a "));
    arduboy.setCursor(24,40);
    print_player_hand(players[1].score);
  }
  else if(substate==152) {  
    if(winner == 0) {
      arduboy.setCursor(24,24);
      for(byte i=0;i<(14-trailing_spaces());i++) {
         arduboy.print(char_state.name[i]);
      }
      if(cripple) {
        arduboy.setCursor(16,32);
        arduboy.print("Crippled Mr Onion!");
      }
      else {
        arduboy.setCursor(54,32);
        arduboy.print("wins!");
      }
      arduboy.setCursor(28,44);
      arduboy.print("You win $");
      arduboy.print(cur_pot);  
    }
    else if(winner == 1) {
      arduboy.setCursor(48,24);
      arduboy.print("Dealer");
      if(cripple) {
        arduboy.setCursor(16,32);
        arduboy.print("Crippled Mr Onion!");
      }
      else {
        arduboy.setCursor(54,32);
        arduboy.print("wins!");
      }
      arduboy.setCursor(28,44);
      arduboy.print("You lose $");
      arduboy.print(cur_pot);
    }
    else {
      arduboy.setCursor(56,32);
      arduboy.print("Tie!");
    }  
  }

   /////////////////////////////////////
   //          Button Handling        //
   /////////////////////////////////////
   if(!ignore_buttons && (substate>=150)) {
       if (arduboy.pressed(A_BUTTON)) {
       ignore_buttons = true;
       ignore_buttons_timer = 25;

          if (substate < 152)
            substate += 1;
          else {
            //Wrap things up and jump to the next state
            if(winner == 0)
              char_state.money += cur_pot*2;
            else if(winner == 2)
              char_state.money += cur_pot;

            if(char_state.money==0)
               char_state.lives -= 1;
            cur_pot  = 0;
            substate = 0;
            if(char_state.lives==0) {
               state = STATE_DIE;
               //You died! Now reset the player and zap the EEPROM settings, just to be evil =)
               char_state.money = 500;
               char_state.type  = random(0,12);   //Choose a random player type
               char_state.name[0] = 'A';
               char_state.name[1] = 'N';
               char_state.name[2] = 'K';
               char_state.name[3] = 'H';
               char_state.name[4] = 'M';
               char_state.name[5] = 'O';
               char_state.name[6] = 'R';
               char_state.name[7] = 'P';
               char_state.name[8] = 'O';
               char_state.name[9] = 'R';
               char_state.name[10]= 'K';
               char_state.name[11]= 'I';
               char_state.name[12]= 'A';
               char_state.name[13]= 'N';
               //burn it with fire!
               EEPROM.write(EEPROM_STORAGE_SPACE_START + 8, (char_state.money & 0xFF));       //money[0]
               EEPROM.write(EEPROM_STORAGE_SPACE_START + 9, (char_state.money & 0xFF00) >> 8); //money[1]
               EEPROM.write(EEPROM_STORAGE_SPACE_START + 10, char_state.type);                //type
               for(int i=0; i<14; i++) {
                  EEPROM.write(EEPROM_STORAGE_SPACE_START + 11 + i, char_state.name[i]);        //name
               }
            }
            else
               state = STATE_MENU;   
          }
       }
    }
    else if (ignore_buttons_timer > 0)
      ignore_buttons_timer -= 1;
    else
      ignore_buttons = false;
}

void do_state_lose() {
  arduboy.drawSlowXYBitmap(8, 4, cmot_bmp, 64, 56, WHITE);
  arduboy.setCursor(72,8);
  arduboy.print(F("Sausages!"));
  arduboy.setCursor(72,16);
  arduboy.print(F("Get your"));
  arduboy.setCursor(72,24);
  arduboy.print(F("sausages"));
  arduboy.setCursor(72,32);
  arduboy.print(F("here!"));
  if(!ignore_buttons) {
     if (arduboy.pressed(A_BUTTON)) {
        ignore_buttons = true;
        ignore_buttons_timer = 25;
        substate = 0;
        state    = STATE_MENU;
        char_state.money += 5;
      }
   }
   else if (ignore_buttons_timer > 0)
      ignore_buttons_timer -= 1;
   else
      ignore_buttons = false;
}

void do_state_die() {
  arduboy.drawSlowXYBitmap(8, 4, death_bmp, 64, 56, WHITE);
  arduboy.setCursor(72,8);
  arduboy.print(F("YOU ARE"));
  arduboy.setCursor(72,16);
  arduboy.print(F("NOT VERY"));
  arduboy.setCursor(72,24);
  arduboy.print(F("GOOD AT"));
  arduboy.setCursor(72,32);
  arduboy.print(F("THIS."));
  if(!ignore_buttons) {
     if (arduboy.pressed(A_BUTTON)) {
        ignore_buttons = true;
        ignore_buttons_timer = 25;
        substate = 0;
        state    = STATE_MENU;
        char_state.money += 5;
      }
   }
   else if (ignore_buttons_timer > 0)
      ignore_buttons_timer -= 1;
   else
      ignore_buttons = false;
}


void setup() {
  Serial.begin(9600);
  state = STATE_START_SCREEN;
  substate = 0;
  // initiate arduboy instance
  arduboy.begin();
  // here we set the framerate to 15, we do not need to run at
  // default 60 and it saves us battery life
  arduboy.setFrameRate(15);
  //Game-specific initialization
  //For test purposes, do the initial deal
  initial_deal();
  ignore_buttons      = false;
  ignore_buttons_timer = 0;

  //Load character settings from EEPROM
  char_state.money = 0;
  char_state.money = (EEPROM.read(EEPROM_STORAGE_SPACE_START + 9) << 8) | EEPROM.read(EEPROM_STORAGE_SPACE_START + 8);  //money
  char_state.type  =  EEPROM.read(EEPROM_STORAGE_SPACE_START + 10);                                                      //type
  for(int i=0; i<14; i++) {
     char_state.name[i] = EEPROM.read(EEPROM_STORAGE_SPACE_START + 11 + i);                                              //name
  }
  char_state.lives = 3;     //We'll be generous and renew the character's lives every time we restart the device
}

void loop() {
  // put your main code here, to run repeatedly:
  arduboy.clear();
  //Draw 'table' border
  arduboy.drawRoundRect(0,0, 128, 64, 4);
  switch(state) {
    case STATE_START_SCREEN:do_state_start_screen();
                            break;
    case STATE_FHI:         do_state_fhi();
                            break;
    case STATE_MENU:        do_state_menu();
                            break;
    case STATE_CHARACTER:   do_state_character();
                            break;
    case STATE_RULES:       do_state_rules();
                            break;
    case STATE_1ST_DRAW:    do_state_1st_draw();
                            break;
    case STATE_BET:         do_state_bet();
                            break;
    case STATE_2ND_DRAW:    do_state_2nd_draw();
                            break;
    case STATE_EVAL:        do_state_eval();
                            break;
    case STATE_LOSE:        do_state_lose();
                            break;
    case STATE_DIE:         do_state_die();
                            break;
    default:break;   
  }
  // then we finally we tell the arduboy to display what we just wrote to the display
  arduboy.display();
}
